const express = require('express');
const mongoose = require('mongoose');
const cors = require('cors');
const bodyParser = require('body-parser');
const config = require('./config/database');
const passport = require('passport');
const path = require('path');
const multer = require('multer');
const UserModel = require('./models/user');
const LeadModel = require('./models/lead');
var fs = require('fs')
const createError = require('http-errors')
const morgan = require('morgan')

// const pdf = require('html-pdf');
// const httpRequest = require('request');

// httpRequest('http://localhost/ssss/test-book/singlePageCode.php?id=5dcbf13341cce36c792ee776&pageNo=141', (error, resposnse, data) => {
//   var html = data;
//   var options = {};
//   pdf.create(html, options).toFile('./businesscard.pdf', function (err, res) {
//     if (err) return console.log(err);
//     console.log(res); // { filename: '/app/businesscard.pdf' }
//   });
// });

mongoose.connect(config.database, {
  useNewUrlParser: true,
  useCreateIndex: true,
  useFindAndModify: false,
  useUnifiedTopology: true,
});

mongoose.connection.on('connected', () => {
  console.log('connected to db ' + config.database);
});

mongoose.connection.on('error', err => {
  console.log('database error:' + err);
});

mongoose.connection.on('disconnected', () => {
  console.log('database disconnected');
});

process.on('SIGINT', async () => {
  await mongoose.connection.close();
  process.exit(0);
})

const app = express();
app.use(cors());
app.use(morgan('dev'));

// create a write stream (in append mode)
var accessLogStream = fs.createWriteStream(path.join(__dirname, 'access.log'), { flags: 'a' })

// setup the logger
// app.use(morgan('combined', { stream: accessLogStream }))

// app.use(morgan('tiny'))

// testing route
const DIR = './uploads';
let storage = multer.diskStorage({
  destination: (req, file, cb) => {
    cb(null, DIR);
  },
  filename: (req, file, cb) => {
    cb(null, file.originalname);
  }
});
let upload = multer({ storage: storage });

app.post('/api/upload', upload.array('file'), function (req, res) {
  // console.log('upload req recieved');
  if (!req.file) {
    // console.log("No file received");
    return res.send({
      success: false
    });

  } else {
    // console.log('file received');
    return res.send({
      success: true
    })
  }
});


app.use(express.json({ limit: '50mb' })); // req.body ---> form data fetch krne ke lie || body parser can also be used in this place.
app.use(express.urlencoded({ extended: true, limit: '50mb' }));

// passport
app.use(passport.initialize());
app.use(passport.session());
require('./config/passport')(passport);
// passport end

// url logging
const jwt = require('jsonwebtoken');

app.use(function (req, res, next) {
  if (req.headers.authorization) {
    const authHeader = (req.headers.authorization).split(' ')[1];
    // verify a token symmetric - synchronous
    var decoded = jwt.verify(authHeader, config.secret);
    if (req.headers.routepath) {
      let fId = req.headers.routepath.split('/');
      if ((fId[fId.length - 1]).length === 24) {
        LeadModel.getLeadById(fId[fId.length - 1], (err, usr) => {
          if (err) throw err;
          if (usr) {
            req.headers.routepath += 'break(name: ' + usr.name + ', mobile: ' + usr.mobile + ')';
            UserModel.updateLastRoute(decoded.id, req.url, req.method, req.headers.routepath);
          } else {
            UserModel.updateLastRoute(decoded.id, req.url, req.method, req.headers.routepath);
          }
        })
      } else {
        UserModel.updateLastRoute(decoded.id, req.url, req.method, req.headers.routepath);
      }
    }
  }
  next();
});

app.use('/api/lms', express.static(path.join(__dirname, 'public', 'lms')));

// parse application/x-www-form-urlencoded
// Body Parser Middleware
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: true }));

// 
const newLead = require('./routes/leads');
const file = require('./routes/file');
app.use('/api/file', file);
app.use('/api/leads', newLead);

// app.use(
//   bodyParser.urlencoded({
//     extended: true
//   })
// );
// app.use(bodyParser.json({ limit: '50mb' }));
// app.use(bodyParser.urlencoded({ limit: '50mb', extended: true }));
// app.use(bodyParser.json());

app.get('/', (req, res) => {
  res.send('Redirect to http://factfinders.in/lms');
});

const user = require('./routes/users');
const office = require('./routes/offices');
const member = require('./routes/members');
const remainder = require('./routes/remainders');
const memberType = require('./routes/member-types');
const followuplogs = require('./routes/follouplogs');
const followUpType = require('./routes/followUpTypes');
const officePackage = require('./routes/office-packages');
const area = require('./routes/areas');
const cities = require('./routes/cities');
const states = require('./routes/states');
const allstates = require('./routes/allStates');
const districts = require('./routes/districts');
const countries = require('./routes/countries');
const request = require('./routes/requests');
const approvedRequest = require('./routes/approvedRequests');
const leadpackage = require('./routes/leadpackages');
const purposes = require('./routes/purposes');
const sources = require('./routes/sources');
const privilageCoupens = require('./routes/privilageCoupens');
const otp = require('./routes/msgOtp');
const roles = require('./routes/roles');
const officesPacks = require('./routes/office-packs');
const incomeRange = require('./routes/income-ranges');
const siteInfo = require('./routes/siteInfos');
const mails = require('./routes/mails');
const activities = require('./routes/activities');
const ssactivities = require('./routes/ssactivities');
const city1 = require("./routes/cities1");

const member1 = require("./routes/members1");
const ssmember1 = require("./routes/ssmembers1");

const state1 = require("./routes/states1");
const country1 = require("./routes/countries1");
const religion = require("./routes/religions");
const caste = require("./routes/castes");
const gotra = require("./routes/gotras");
const motherTongue = require("./routes/motherTongues");
const collegePassout = require("./routes/collegePassouts");
const educationDetails = require("./routes/educationDetails");
const familyIncome = require("./routes/family-incomes");
const familyStatus = require("./routes/family-status");
const familyValue = require("./routes/family-values");
const professionalStatus = require("./routes/professional-status");
const occupation = require("./routes/occupations");
const promoPage = require("./routes/promotional-pages");
const media = require("./routes/medias");
const namaj = require("./routes/namaj");
const rashi = require("./routes/rashis");
const face = require("./routes/faceCovers");
const beard = require("./routes/beards");
const counters = require("./routes/counters");
const new_member = require("./routes/new_members");
const religion_masters = require("./routes/religion-masters");
const new_religion_masters = require("./routes/new-religion-masters");
const member_master = require("./routes/member-masters");
const event = require("./routes/events");
const ad = require("./routes/ads");
const subscriptionPlan = require("./routes/subscription-plans");
const bookSubscriptionPlan = require("./routes/book-subscription-plans");
const mainOtp = require("./routes/otps");
const ssTransaction = require("./routes/ss-transactions");
const transactions = require("./routes/transactions");
const desired = require("./routes/ss-desireds");
const oldDesired = require("./routes/old_desireds");
const ssNotifications = require("./routes/ss_notifications");
const chatrooms = require("./routes/chat-rooms");
const chats = require("./routes/chats");
const companies = require("./routes/companies");
const bankAccounts = require("./routes/bank-accounts");
const transactionTypes = require("./routes/transaction-types");
const leadTransactions = require("./routes/lead-transactions");
const leadRemainders = require("./routes/lead-remainders");
const leadServices = require("./routes/leadServices");
const leadBillings = require("./routes/leadBillings");

app.use('/api/areas', area);
app.use('/api/approvedrequests', approvedRequest);
app.use('/api/requests', request);
app.use('/api/remainders', remainder);
app.use('/api/officepackages', officePackage);
app.use('/api/followuptypes', followUpType);
app.use('/api/followuplogs', followuplogs);
app.use('/api/membertypes', memberType);
app.use('/api/packages', leadpackage);
app.use('/api/members', member);
app.use('/api/office', office);
app.use('/api/districts', districts);
app.use('/api/countries', countries);
app.use('/api/cities', cities);
app.use('/api/states', states);
app.use('/api/allstates', allstates);
app.use('/api/user', user);
app.use('/api/purposes', purposes);
app.use('/api/sources', sources);
app.use('/api/privilageCoupens', privilageCoupens);
app.use('/api/otp', otp);
app.use('/api/roles', roles);
app.use('/api/officesPacks', officesPacks);
app.use('/api/income-range', incomeRange);
app.use('/api/siteInfo', siteInfo);
app.use('/api/mail', mails);
app.use('/api/activities', activities);
app.use('/api/ssactivities', ssactivities);
app.use("/api/city", city1);

app.use("/api/member", member1);
app.use("/api/ssmember", ssmember1);

app.use("/api/state", state1);
app.use("/api/country", country1);
app.use("/api/religion", religion);
app.use("/api/caste", caste);
app.use("/api/gotra", gotra);
app.use("/api/motherTongue", motherTongue);
app.use("/api/collegePassout", collegePassout);
app.use("/api/educationDetails", educationDetails);
app.use("/api/familyIncome", familyIncome);
app.use("/api/familyStatus", familyStatus);
app.use("/api/familyValue", familyValue);
app.use("/api/professionalStatus", professionalStatus);
app.use("/api/occupation", occupation);
app.use("/api/promotionalPage", promoPage);
app.use("/api/media", media);
app.use("/api/namaj", namaj);
app.use("/api/faceCover", face);
app.use("/api/beard", beard);
app.use("/api/rashi", rashi);
app.use("/api/counters", counters);

app.use("/api/new_member", new_member);
app.use("/api/religion-master", religion_masters);
app.use("/api/new-religion-master", new_religion_masters);
app.use("/api/member-master", member_master);
app.use("/api/event", event);
app.use("/api/ad-master", ad);
app.use("/api/subscription-plan", subscriptionPlan);
app.use("/api/book-subscription-plan", bookSubscriptionPlan);
app.use("/api/main-otp", mainOtp);
app.use("/api/ss-transaction", ssTransaction);
app.use("/api/transactions", transactions);
app.use("/api/ss-desired", desired);
app.use("/api/old-desired", oldDesired);
app.use("/api/ss-notifications", ssNotifications);
app.use("/api/chat-room", chatrooms);
app.use("/api/chat", chats);
app.use("/api/companies", companies);
app.use("/api/bankAccounts", bankAccounts);
app.use("/api/transactionTypes", transactionTypes);
app.use("/api/leadTransactions", leadTransactions);
app.use("/api/leadRemainders", leadRemainders);
app.use("/api/leadServices", leadServices);
app.use("/api/leadBillings", leadBillings);

var commonRouter = require('./apiv2/routes/common');
app.use('/api/common', commonRouter);

app.get('/restart-facetest3', (req, res) => {
  restartFaceTest3();
  res.send('Restarting facetest3...');
});

app.use(async (req, res, next) => {
  next(createError.NotFound())
})

app.use((err, req, res, next) => {
  res.status(err.status || 500)
  res.send({
    error: {
      status: err.status || 500,
      message: err.message,
    },
  })
})


port = 3400;
host = '0.0.0.0';

const http = app.listen(port, host, () => {
  console.log('Server is running in port: ' + port);
});

// const io = require('socket.io')(http)

// module.exports = io;

// const socketController = require('./controllers/socketController');
var pyProg;
function runfaceDetection() {
  console.log('run python');
  const { spawn } = require('child_process');
  pyProg = spawn('python3', ['./facetest3.py']);

  pyProg.stdout.on('data', function (data) {

    console.log(data.toString());
  });
  pyProg.stdout.on('error', function (data) {
    console.log('restarted');
    runfaceDetection();
  });
}

function restartFaceTest3() {
  if (pyProg) {
    console.log('Killing existing facetest3 process...');
    pyProg.kill();
  }

  runfaceDetection();
}

runfaceDetection();

// Handle server shutdown
process.on('SIGINT', () => {
  http.close(() => {
    console.log('Server has been terminated');
    process.exit(0);
  });
});