const caste = require("./../models/caste");
const rc = require("./responseController");

module.exports.add = (req, res) => {
  let newCaste = new caste({
    caste_id: req.body.caste_id,
    caste_name: req.body.caste_name,
    caste_slug: req.body.caste_slug,
    caste_parent: req.body.caste_parent,
    caste_added_by: req.body.caste_added_by,
    caste_is_deleted: req.body.caste_is_deleted,
    caste_order: req.body.caste_order,
    caste_status: req.body.caste_status,
    caste_religion_name: req.body.caste_religion_name
  });
  caste.addCaste(newCaste, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      rc.setResponse(res, {
        success: true,
        msg: "Caste added successfully",
        data: data
      });
    }
  });
};

module.exports.get = (req, res) => {
  // Initialize all inputs
  const id = req.body._id;
  caste.getCaste(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id"
        });
      }
    }
  });
};

module.exports.getAll = (req, res) => {
  caste.getAllCaste((err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "All data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data present"
        });
      }
    }
  });
};
module.exports.update = (req, res) => {
  const id = req.body._id;

  caste.getCaste(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        let isChange = false;
        data.lastChanges = {};

        if (req.body.caste_name) {
          isChange = true;
          data.lastChanges.caste_name = data.caste_name;
          data.caste_name = req.body.caste_name;
        }

        if (req.body.caste_slug) {
          isChange = true;
          data.lastChanges.caste_slug = data.caste_slug;
          data.caste_slug = req.body.caste_slug;
        }
        if (req.body.caste_parent) {
          isChange = true;
          data.lastChanges.caste_parent = data.caste_parent;
          data.caste_parent = req.body.caste_parent;
        }

        if (req.body.caste_is_deleted) {
          isChange = true;
          data.lastChanges.caste_is_deleted = data.caste_is_deleted;
          data.caste_is_deleted = req.body.caste_is_deleted;
        }

        if (req.body.caste_order) {
          isChange = true;
          data.lastChanges.caste_order = data.caste_order;
          data.caste_order = req.body.caste_order;
        }
        if (req.body.caste_status) {
          isChange = true;
          data.lastChanges.caste_status = data.caste_status;
          data.caste_status = req.body.caste_status;
        }
        if (req.body.caste_religion_name) {
          isChange = true;
          data.lastChanges.caste_religion_name = data.caste_religion_name;
          data.caste_religion_name = req.body.caste_religion_name;
        }
        if (isChange) {
          console.log(data);
          data.caste_updated_on = Date.now();
          caste.addCaste(data, (err, newData) => {
            if (err) {
              rc.setResponse(res, {
                msg: err.message,
                data: err
              });
            } else {
              rc.setResponse(res, {
                success: true,
                msg: "Details updated successfully",
                data: newData
              });
            }
          });
        } else {
          rc.setResponse(res, {
            msg: "No data to update"
          });
        }
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id"
        });
      }
    }
  });
};

module.exports.delete = (req, res) => {
  const id = req.body._id;
  caste.deleteCaste(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "Data deleted successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id "
        });
      }
    }
  });
};
