const Event = require("./../models/event");
const rc = require("./responseController");
const newMem = require("./../models/new_member");
const Plan = require("./../models/subscription-plan");
// const Transaction = require("./../models/ss-transaction");
const Transaction = require("./../models/transaction");
const _ = require('lodash');
const fs = require('fs');

module.exports.add = (req, res) => {
  let newEvent = new Event({
    Title: req.body.Title,
    shortTitle: req.body.shortTitle,
    description: req.body.description,
    hindiTitle: req.body.hindiTitle,
    hindiShortTitle: req.body.hindiShortTitle,
    hindiDescription: req.body.hindiDescription,
    live: req.body.live,
    running: req.body.running,
    religion: req.body.religion,
    caste: req.body.caste,
    incomeRange: req.body.incomeRange,
    area: req.body.area,
    firstCategory: req.body.firstCategory,
    secondCategory: req.body.secondCategory,
    thirdCategory: req.body.thirdCategory,
    bookData: req.body.bookData,
    limit: req.body.limit,
    softBookMrp: req.body.softBookMrp,
    softBookPrice: req.body.softBookPrice,
    hardBookMrp: req.body.hardBookMrp,
    hardBookPrice: req.body.hardBookPrice,
    startedAt: req.body.startedAt,
    endedAt: req.body.endedAt,
    members: req.body.members
  });
  Event.addEvent(newEvent, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      rc.setResponse(res, {
        success: true,
        msg: "Event added successfully",
        data: data
      });
    }
  });
};

module.exports.get = (req, res) => {
  // Initialize all inputs
  const id = req.body._id;
  Event.getEvent(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id"
        });
      }
    }
  });
};

module.exports.getEventById = (req, res) => {
  // Initialize all inputs
  const id = req.params.id;
  Event.getEvent(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id"
        });
      }
    }
  });
};

module.exports.getEventsByIds = (req, res) => {
  // Initialize all inputs
  const ids = req.body.ids;
  Event.getEventsByIds(ids, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data.length) {
        rc.setResponse(res, {
          success: true,
          msg: "data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id"
        });
      }
    }
  });
};

module.exports.getAll = (req, res) => {
  Event.getAllEvents((err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "All data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data present"
        });
      }
    }
  });
};

module.exports.getUpcomingEvents = (req, res) => {
  Event.getAllUpcomingEvents((err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "All data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data present"
        });
      }
    }
  });
};

module.exports.getEndedEvents = (req, res) => {
  Event.getAllEndedEvents((err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "All data fetched successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data present"
        });
      }
    }
  });
};

module.exports.subscribeMembers = (req, res) => {
  const members = req.body.members;
  const event = req.body.event;
  const plan = req.body.plan;
  Event.getEvent(event, (err, fetchedEvent) => {
    if (err) {
      rc.setResponse(res, {
        msg: "Error in fetching event"
      });
    } else {
      Plan.getPlan(plan, (err, fetchedPlan) => {
        if (err) {
          rc.setResponse(res, {
            msg: "Error in fetching plan"
          });
        } else {
          for (const member of members) {
            newMem.getMember(member, (err, data) => {
              if (err) {
                rc.setResponse(res, {
                  msg: "Error in fetching member",
                  data: err
                });
              } else {
                data.subscribed_events.push(event);
                let oldWallet = data.wallet;
                newWallet = oldWallet - fetchedPlan.plan_price;
                data.wallet = newWallet;
                let transactionToAdd = {
                  payee: data.member_id + '@' + data._id,
                  receiver: 0,
                  from: 'member',
                  to: 'company',
                  action: 'debit',
                  amount: fetchedPlan.plan_price,
                  prevBal: data.wallet,
                  Remark: 'A: Member Subscription',
                  Refrence: fetchedPlan.name + '@' + fetchedPlan._id,
                  Date: Date.now(),
                  created_at: Date.now(),
                  created_by: req.user ? req.user.username : ''
                }
                Transaction.newTransaction(transactionToAdd, (transdata) => {
                  // console.log(transdata);
                  return true;
                });
                // let newTransaction = new Transaction({
                //   member_id: data,
                //   amount: fetchedPlan.plan_price,
                //   transaction: "Debit",
                //   remark: `${fetchedPlan.plan_price} debited for event ${fetchedEvent.Title} from ${data.member_name}'s wallet `,
                //   event_id: event,
                //   plan_id: plan
                // });
                // Transaction.addTransaction(newTransaction, (err, trans) => {
                //   if (err) {
                //     rc.setResponse(res, {
                //       msg: "Error in making transaction",
                //       data: err
                //     });
                //   } else {
                newMem.addNewMember(data, (err, updatedData) => {
                  if (err) {
                    rc.setResponse(res, {
                      msg: "Error in updating member",
                      data: err
                    });
                  } else {
                    let newMemPlan = {
                      member_id: member,
                      plan_id: plan
                    };
                    fetchedEvent.members.push(newMemPlan);
                    Event.addEvent(fetchedEvent, (err, newData) => {
                      if (err) {
                        // rc.setResponse(res,{
                        //   msg: 'Error in adding member and plan in event',
                        //   data: err
                        // })
                      } else {
                        rc.setResponse(res, {
                          success: true,
                          msg:
                            "Event added in all members and members and plan added in selected event"
                        });
                      }
                    });
                  }
                });
                // }
                // });
              }
            });
          }
        }
      });
    }
  });
};

module.exports.subscribeMembersWithMemberId = (req, res) => {
  const members = req.body.members;
  const event = req.body.event;
  const plan = req.body.plan;
  Event.getEvent(event, (err, fetchedEvent) => {
    if (err) {
      rc.setResponse(res, {
        msg: "Error in fetching event"
      });
    } else {
      Plan.getPlan(plan, (err, fetchedPlan) => {
        if (err) {
          rc.setResponse(res, {
            msg: "Error in fetching plan"
          });
        } else {
          rc.setResponse(res, {
            success: true,
            msg:
              "Event added in all members and members and plan added in selected event"
          });
          for (const member of members) {
            newMem.searchByMemberId(member, (err, data) => {
              if (err) {
                // rc.setResponse(res, {
                //   msg: "Error in fetching member",
                //   data: err
                // });
                console.log({
                  msg: "Error in fetching member",
                  data: err
                });
              } else {
                data.subscribed_events.push(event);
                let oldWallet = data.wallet;
                newWallet = oldWallet - fetchedPlan.plan_price;
                data.wallet = newWallet;

                let transactionToAdd = {
                  payee: data.member_id + '@' + data._id,
                  receiver: 0,
                  from: 'member',
                  to: 'company',
                  action: 'debit',
                  amount: fetchedPlan.plan_price,
                  prevBal: data.wallet,
                  Remark: 'A: Member Subscription',
                  Refrence: fetchedPlan.name + '@' + fetchedPlan._id,
                  Date: Date.now(),
                  created_at: Date.now(),
                  created_by: req.user ? req.user.username : ''
                }
                Transaction.newTransaction(transactionToAdd, (transdata) => {
                  console.log(transdata);
                  return true;
                });

                // let newTransaction = new Transaction({
                //   member_id: data,
                //   amount: fetchedPlan.plan_price,
                //   transaction: "Debit",
                //   remark: `${fetchedPlan.plan_price} debited for event ${fetchedEvent.Title} from ${data.member_name}'s wallet `,
                //   event_id: event,
                //   plan_id: plan
                // });
                // Transaction.addTransaction(newTransaction, (err, trans) => {
                //   if (err) {
                //     // rc.setResponse(res, {
                //     //   msg: "Error in making transaction",
                //     //   data: err
                //     // });
                //     console.log({
                //       msg: "Error in making transaction",
                //       data: err
                //     });
                //   } else {
                newMem.addNewMember(data, (err, updatedData) => {
                  if (err) {
                    // rc.setResponse(res, {
                    //   msg: "Error in updating member",
                    //   data: err
                    // });
                    console.log({
                      msg: "Error in updating member",
                      data: err
                    });
                  } else {
                    let newMemPlan = {
                      member_id: member,
                      plan_id: plan
                    };
                    fetchedEvent.members.push(newMemPlan);
                    Event.addEvent(fetchedEvent, (err, newData) => {
                      if (err) {
                        // rc.setResponse(res,{
                        //   msg: 'Error in adding member and plan in event',
                        //   data: err
                        // })
                        console.log({
                          msg: 'Error in adding member and plan in event',
                          data: err
                        })
                      } else {
                        // rc.setResponse(res, {
                        //   success: true,
                        //   msg:
                        //     "Event added in all members and members and plan added in selected event"
                        // });
                      }
                    });
                  }
                });
                //   }
                // });
              }
            });
          }
        }
      });
    }
  });
};

module.exports.subscribeSingleMember = (req, res) => {
  const member = req.body.member_id;
  const event = req.body.event_id;
  const plan = req.body.plan_id;
  console.log(req.body.event_id);
  Event.getEvent(event, (err, fetchedEvent) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (fetchedEvent) {
        let newSub = {
          member_id: member,
          plan_id: plan
        };
        console.log(newSub);

        fetchedEvent.members.push(newSub);
        // console.log(fetchedEvent);
        Event.addEvent(fetchedEvent, (err, updatedEvent) => {
          if (err) {
            rc.setResponse(res, {
              msg: err.message,
              data: err
            });
          } else {
            rc.setResponse(res, {
              success: true,
              msg: "Event updated successfully",
              data: updatedEvent
            });
          }
        });
      }
    }
  });
  // Event.getEvent(event, (err, fetchedEvent) => {
  //   if (err) {
  //     rc.setResponse(res, {
  //       msg: "Error in fetching event"
  //     });
  //   } else {
  //     if (fetchedEvent) {
  //       newMem.getMember(member, (err, data) => {
  //         if (err) {
  //           rc.setResponse(res, {
  //             msg: "Error in fetching member",
  //             data: err
  //           });
  //         } else {
  //           if (data) {
  //             data.subscribed_events.push(event);
  //             newMem.addNewMember(data, (err, updatedData) => {
  //               console.log(updatedData);
  //               if (err) {
  //                 rc.setResponse(res, {
  //                   msg: "Error in updating member",
  //                   data: err
  //                 });
  //               } else {
  //                 if (updatedData) {
  //                   let newMemPlan = {
  //                     member_id: member,
  //                     plan_id: plan
  //                   };
  //                   fetchedEvent.members.push(newMemPlan);
  //                   Event.addEvent(fetchedEvent, (err, newData) => {
  //                     if (err) {
  //                       rc.setResponse(res, {
  //                         msg: "Error in adding member and plan in event",
  //                         data: err
  //                       });
  //                     } else {
  //                       rc.setResponse(res, {
  //                         success: true,
  //                         msg: "Member added and subscribed successfully"
  //                       });
  //                     }
  //                   });
  //                 }
  //               }
  //             });
  //           }
  //         }
  //       });
  //     }
  //   }
  // });
};

module.exports.unsubscribeMembers = (req, res) => {
  const members = req.body.members;
  const event = req.body.event;
  for (const member of members) {
    // newMem.searchByMemberId(member, (err, fetchedMember) => {
    newMem.getMember(member, (err, fetchedMember) => {
      if (err) {
        console.log({
          msg: "Error in getting members data",
          data: err.message
        });
      } else {
        // console.log(fetchedMember);
        if (fetchedMember) {
          let memberSubscribedEvents = [];
          memberSubscribedEvents = fetchedMember.subscribed_events;
          // console.log(fetchedMember.subscribed_events);
          for (let i = 0; i < memberSubscribedEvents.length; i++) {
            const memberEvent = memberSubscribedEvents[i];
            // console.log(memberEvent, event);
            if (memberEvent === event) {
              memberSubscribedEvents.splice(i, 1);
            }
          }
          fetchedMember.subscribed_events = memberSubscribedEvents;

          newMem.addNewMember(fetchedMember, (err, updatedMember) => {
            if (err) {
              console.log({
                msg: "Error in unsubscribing member from event",
                data: err.message
              });
            } else {
              if (updatedMember) {
                Event.getEvent(event, (err, fetchedEvent) => {
                  if (err) {
                    console.log({
                      msg: "error in fetching event to update",
                      data: err.message
                    });
                  } else {
                    if (fetchedEvent) {
                      let memberObjects = fetchedEvent.members;
                      for (let i = 0; i < memberObjects.length; i++) {
                        const memberObject = memberObjects[i];
                        // console.log(updatedMember._id);
                        // console.log(memberObject.member_id);
                        if (memberObject.member_id == updatedMember._id) {
                          memberObjects.splice(i, 1);
                        }
                      }
                      Event.addEvent(
                        fetchedEvent,
                        (err, updatedEvent) => {
                          if (err) {
                            console.log(res, {
                              msg: "error in fetching event to update",
                              data: err.message
                            });
                          }
                        }
                      );
                      fetchedEvent.members = memberObjects;

                      // Transaction.getTransByMemEvent(
                      //   fetchedMember._id,
                      //   fetchedEvent._id,
                      //   (err, fetchedTrans) => {
                      //     if (err) {
                      //       console.log({
                      //         msg: "error in fetching transaction",
                      //         data: err.message
                      //       });
                      //     } else {
                      //       if (fetchedTrans) {
                      //         // console.log(fetchedTrans);
                      //         subscribedPlan = fetchedTrans.amount;
                      //         let newTrans = new Transaction({
                      //           member_id: fetchedMember,
                      //           amount: subscribePlan,
                      //           transaction: "Credit",
                      //           remark: `Rs. ${subscribePlan} credit to ${fetchedMember}'s wallet`,
                      //           event_id: fetchedEvent._id,
                      //           plan_id: fetchedPlan._id
                      //         });
                      //         Transaction.addTransaction(
                      //           newTrans,
                      //           (err, trans) => {
                      //             if (err) {
                      //               console.log({
                      //                 msg: "error in fetching transaction",
                      //                 data: err.message
                      //               });
                      //             } else {
                      //               if (trans) {
                      //                 Event.addEvent(
                      //                   fetchedEvent,
                      //                   (err, updatedEvent) => {
                      //                     if (err) {
                      //                       console.log({
                      //                         msg:
                      //                           "error in fetching event to update",
                      //                         data: err.message
                      //                       });
                      //                     }
                      //                   }
                      //                 );
                      //               }
                      //             }
                      //           }
                      //         );
                      //       } else {
                      //         Event.addEvent(
                      //           fetchedEvent,
                      //           (err, updatedEvent) => {
                      //             if (err) {
                      //               console.log(res, {
                      //                 msg: "error in fetching event to update",
                      //                 data: err.message
                      //               });
                      //             }
                      //           }
                      //         );
                      //       }
                      //     }
                      //   }
                      // );
                    }
                  }
                });
              }
            }
          });
        } else {
          console.log('member not found: ' + member);
        }
      }
    });
  }
  rc.setResponse(res, {
    success: true,
    msg: "All members unsubscribed from event successfully"
  });
};

module.exports.update = (req, res) => {
  const id = req.body._id;

  Event.getEvent(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        let isChange = false;
        if (req.body.Title) {
          isChange = true;
          data.Title = req.body.Title;
        }
        if (req.body.shortTitle) {
          isChange = true;
          data.shortTitle = req.body.shortTitle;
        }
        if (req.body.description) {
          isChange = true;
          data.description = req.body.description;
        }
        if (req.body.hindiTitle) {
          isChange = true;
          data.hindiTitle = req.body.hindiTitle;
        }
        if (req.body.hindiShortTitle) {
          isChange = true;
          data.hindiShortTitle = req.body.hindiShortTitle;
        }
        if (req.body.hindiDescription) {
          isChange = true;
          data.hindiDescription = req.body.hindiDescription;
        }
        if (req.body.live !== data.live) {
          isChange = true;
          data.live = req.body.live;
        }
        if (req.body.bookData) {
          isChange = true;
          data.bookData = req.body.bookData;
        }
        if (req.body.firstCategory != data.firstCategory) {
          isChange = true;
          data.firstCategory = req.body.firstCategory;
        }
        if (req.body.secondCategory != data.secondCategory) {
          isChange = true;
          data.secondCategory = req.body.secondCategory;
        }
        if (req.body.thirdCategory != data.thirdCategory) {
          isChange = true;
          data.thirdCategory = req.body.thirdCategory;
        }
        if (req.body.running) {
          isChange = true;
          data.running = req.body.running;
        }
        if (req.body.religion != data.religion) {
          isChange = true;
          data.religion = req.body.religion;
          console.log(data.religion);
        }
        if (req.body.caste) {
          isChange = true;
          data.caste = req.body.caste;
        }
        if (req.body.incomeRange) {
          isChange = true;
          data.incomeRange = req.body.incomeRange;
        }
        if (req.body.area) {
          isChange = true;
          data.area = req.body.area;
        }
        if (req.body.limit) {
          isChange = true;
          data.limit = req.body.limit;
        }
        if (req.body.softBookMrp != data.softBookMrp) {
          isChange = true;
          data.softBookMrp = req.body.softBookMrp;
        }
        if (req.body.softBookPrice != data.softBookPrice) {
          isChange = true;
          data.softBookPrice = req.body.softBookPrice;
        }
        if (req.body.hardBookMrp != data.hardBookMrp) {
          isChange = true;
          data.hardBookMrp = req.body.hardBookMrp;
        }
        if (req.body.hardBookPrice != data.hardBookPrice) {
          isChange = true;
          data.hardBookPrice = req.body.hardBookPrice;
        }
        if (req.body.startedAt) {
          isChange = true;
          data.startedAt = req.body.startedAt;
        }
        if (req.body.endedAt) {
          isChange = true;
          data.endedAt = req.body.endedAt;
        }
        if (req.body.createdAt) {
          isChange = true;
          data.createdAt = req.body.createdAt;
        }
        if (req.body.updatedAt) {
          isChange = true;
          data.updatedAt = req.body.updatedAt;
        }
        if (req.body.members) {
          isChange = true;
          data.members = req.body.members;
        }
        if (isChange) {
          Event.addEvent(data, (err, newData) => {
            if (err) {
              rc.setResponse(res, {
                msg: err.message,
                data: err
              });
            } else {
              rc.setResponse(res, {
                success: true,
                msg: "Details updated successfully",
                data: newData
              });
            }
          });
        } else {
          rc.setResponse(res, {
            msg: "No data to update"
          });
        }
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id"
        });
      }
    }
  });
};

module.exports.generateEventBookSorting = (req, res) => {
  const evId = req.params.id;
  res.json({ success: true, msg: 'sorting...' });
  let fc = [];
  let sc = [];
  let tc = [];
  let sortedMembers = [...fc, ...sc, ...tc];

  Event.getEvent(evId, (err, event) => {
    if (err) {
      console.log(err);
      return;
    }
    console.log('event logged');
    newMem.getMemInEvent(evId, (err, members) => {
      if (err) {
        console.log(err);
        return;
      }
      console.log('mems logged');
      fc = _.sortBy(members, ['member_gender', getFieldName(event.firstCategory.title), getFieldName(event.secondCategory.title), getFieldName(event.thirdCategory.title)]);
      // for (const item of fc) {
      //   console.log(item[getFieldName(event.firstCategory.title)]);
      //   console.log(item[getFieldName(event.secondCategory.title)]);
      //   console.log(item[getFieldName(event.thirdCategory.title)]);
      // }

      Event.updateBookSortedMembers(evId, fc, (err, data) => {
        console.log('Members Sorted');
      });
    });
  });
}

function getFieldName(name) {
  switch (name) {
    case 'Qualification':
      return 'member_education_details';

    case 'Profession':
      return 'member_professional_status'

    case 'Religion':
      return 'member_religion_name'

    case 'Caste':
      return 'member_caste_name'

    case 'Sub-Caste':
      return 'member_caste_name'

    case 'Gotra':
      return 'member_gotra'

    default:
      return 'member_gender';
  }
}


// module.exports.createEventBookIndexes = (req, res) => {
//   const evId = req.params.id;
//   res.json({ success: true, msg: 'indexing...' });
//   let pages = [];
//   Event.getEvent(evId, (err, event) => {
//     if (err) {
//       console.log(err);
//       return;
//     }
//     console.log('event logged');
//     const members = event.sortedMembersForBook;
//     Plan.getPlansByEvent(evId, (err, plans) => {
//       if (err) {
//         console.log(err);
//         return;
//       }
//       console.log('plans logged');
//     });
//     eventMembers = [];
//     notfound = [];
//     for (const mem of members) {
//       let found = false;
//       for (const evm of event.members) {
//         if (evm.member_id == mem._id) {
//           found = true;
//           eventMembers.push(mem);
//         }
//       }
//       if (!found) {
//         notfound.push(mem);
//         // console.log(mem);
//       }
//     }
//     eventMembers = _.uniqBy(eventMembers, "member_id")
//     notfound = _.uniqBy(notfound, "member_id")
//     console.log(eventMembers.length + notfound.length);
//     console.log(notfound.length);
//     console.log(members.length);
//     console.log(event.members.length);
//     console.log(event.members.length - members.length);
//   });
// }

// module.exports.createEventBookIndexes = (req, res) => {
//   const evId = req.params.id;
//   res.json({ success: true, msg: 'indexing...' });
//   let pages = [];
//   Event.getEvent(evId, (err, event) => {
//     if (err) {
//       console.log(err);
//       return;
//     }
//     console.log('event logged');
//     const members = event.sortedMembersForBook;
//     Plan.getPlansByEvent(evId, (err, plans) => {
//       if (err) {
//         console.log(err);
//         return;
//       }
//       console.log('plans logged');
//       const pageProfiles = [];
//       let pageChange = false;
//       let memGender = 'Female';
//       for (const mem of members) {
//         let found = false;
//         for (const evm of event.members) {
//           if (evm.member_id == mem._id) {
//             if (memGender !== mem.member_gender) {
//               memGender = mem.member_gender;
//               pageChange = true;
//             }
//             found = true;
//             for (const plan of plans) {
//               if (plan._id == evm.plan_id) {
//                 pageProfiles.push(
//                   {
//                     weight: getPlanWeight(plan.size),
//                     pageProfile: mem.member_id,
//                     skip: pageChange
//                   }
//                 )
//                 pageChange = false;
//               }
//             }
//           }
//         }
//         if (!found) {
//           if (memGender !== mem.member_gender) {
//             memGender = mem.member_gender;
//             pageChange = true;
//           }
//           pageProfiles.push(
//             {
//               weight: 1,
//               pageProfile: mem.member_id,
//               skip: pageChange
//             }
//           )
//           pageChange = false;
//         }
//       }
//       const uniqPageProfiles = _.uniqBy(pageProfiles, 'pageProfile');
//       console.log('Total Profiles: ' + uniqPageProfiles.length);
//       // for (const upp of uniqPageProfiles) {
//       //   console.log(upp);
//       // }

//       const pages = [];
//       let inserted = 0;
//       while (inserted < uniqPageProfiles.length) {
//         let page = {
//           pageWeight: 0,
//           pageProfiles: []
//         }
//         let skip = false;
//         while (!skip && page.pageWeight < 4) {
//           if (page.pageWeight == 0 && uniqPageProfiles[inserted] && uniqPageProfiles[inserted].skip) {
//             if ((page.pageWeight + uniqPageProfiles[inserted].weight) <= 4) {
//               page.pageWeight += uniqPageProfiles[inserted].weight;
//               page.pageProfiles.push(uniqPageProfiles[inserted]);
//               inserted++;
//             } else {
//               skip = true;
//             }
//           } else if (uniqPageProfiles[inserted] && !uniqPageProfiles[inserted].skip) {
//             if ((page.pageWeight + uniqPageProfiles[inserted].weight) <= 4) {
//               page.pageWeight += uniqPageProfiles[inserted].weight;
//               page.pageProfiles.push(uniqPageProfiles[inserted]);
//               inserted++;
//             } else {
//               skip = true;
//             }
//           } else {
//             break;
//           }
//         }
//         // console.log(page);
//         pages.push(page);
//       }
//       console.log(pages.length);
//       // let total = 0;
//       // for (const page of pages) {
//       //   total += page.pageProfiles.length;
//       // }
//       // console.log(total);

//       Event.updateBook(evId, pages, (err, data) => {
//         if (err) {
//           console.log(err);
//         }
//         // console.log(data);
//         console.log('Pages Generated');
//         console.log(pages.length);
//         let total = 0;
//         for (const page of pages) {
//           total += page.pageProfiles.length;
//         }
//         console.log(total);
//         console.log(members.length);
//         console.log(members.length - total);
//       });
//     })
//     // })
//   })
// }

module.exports.createEventBookIndexes = (req, res) => {
  const evId = req.params.id;
  res.json({ success: true, msg: 'indexing...' });
  let pages = [];
  Event.getEvent(evId, (err, event) => {
    if (err) {
      console.log(err);
      return;
    }
    console.log('event logged');
    const members = event.sortedMembersForBook;
    Plan.getPlansByEvent(evId, (err, plans) => {
      if (err) {
        console.log(err);
        return;
      }
      console.log('plans logged');
      const pageProfiles = [];
      let pageChange = false;
      let memGender = 'Female';
      let fc = '';
      for (const mem of members) {
        let found = false;
        for (const evm of event.members) {
          if (evm.member_id == mem._id) {
            if (memGender !== mem.member_gender) {
              memGender = mem.member_gender
              pageChange = true;
            }
            if (fc !== mem[getFieldName(event.firstCategory.title)]) {
              fc = mem[getFieldName(event.firstCategory.title)];
              pageChange = true;
            }
            found = true;
            for (const plan of plans) {
              if (plan._id == evm.plan_id) {
                pageProfiles.push(
                  {
                    weight: getPlanWeight(plan.size),
                    pageProfile: mem.member_id,
                    skip: pageChange
                  }
                )
                pageChange = false;
              }
            }
          }
        }
        if (!found) {
          if (memGender !== mem.member_gender) {
            memGender = mem.member_gender
            pageChange = true;
          }
          if (fc !== mem[getFieldName(event.firstCategory.title)]) {
            fc = mem[getFieldName(event.firstCategory.title)];
            pageChange = true;
          }
          pageProfiles.push(
            {
              weight: 1,
              pageProfile: mem.member_id,
              skip: pageChange
            }
          )
          pageChange = false;
        }
      }
      const uniqPageProfiles = _.uniqBy(pageProfiles, 'pageProfile');
      console.log('Total Profiles: ' + uniqPageProfiles.length);
      // for (const upp of uniqPageProfiles) {
      //   console.log(upp);
      // }

      const pages = [];
      let inserted = 0;
      while (inserted < uniqPageProfiles.length) {
        let page = {
          pageWeight: 0,
          pageProfiles: []
        }
        let skip = false;
        while (!skip && page.pageWeight < 4) {
          // if (page.pageWeight == 0 && uniqPageProfiles[inserted] && uniqPageProfiles[inserted].skip) {
          if (page.pageWeight == 0 && uniqPageProfiles[inserted]) {
            // if ((page.pageWeight + uniqPageProfiles[inserted].weight) <= 4) {
            page.pageWeight += uniqPageProfiles[inserted].weight;
            page.pageProfiles.push(uniqPageProfiles[inserted]);
            inserted++;
            if (!((page.pageWeight + uniqPageProfiles[inserted].weight) <= 4)) {
              // } else {
              skip = true;
            }
          } else if (uniqPageProfiles[inserted] && !uniqPageProfiles[inserted].skip) {
            if ((page.pageWeight + uniqPageProfiles[inserted].weight) <= 4) {
              page.pageWeight += uniqPageProfiles[inserted].weight;
              page.pageProfiles.push(uniqPageProfiles[inserted]);
              inserted++;
            } else {
              skip = true;
            }
          } else {
            console.log(uniqPageProfiles[inserted]);
            break;
          }
          console.log(uniqPageProfiles[inserted]);
        }
        // console.log(page);
        pages.push(page);
      }
      console.log(pages.length);
      // let total = 0;
      // for (const page of pages) {
      //   total += page.pageProfiles.length;
      // }
      // console.log(total);

      Event.updateBook(evId, pages, (err, data) => {
        if (err) {
          console.log(err);
        }
        // console.log(data);
        console.log('Pages Generated');
        console.log(pages.length);
        let total = 0;
        for (const page of pages) {
          total += page.pageProfiles.length;
        }
        console.log(total);
        console.log(members.length);
        console.log(members.length - total);
      });
    })
    // })
  })
}

function getPlanWeight(size) {
  let weight = 1;
  switch (size) {
    case '1/2':
      weight = 2;
      break;
    case '1':
      weight = 4;
      break;
    case '1/4 + 1':
      weight = 5;
      break;
    case '1/2 + 1':
      weight = 6;
      break;
    case '1 + 1':
      weight = 8;
      break;
  }
  return weight;
}

// module.exports.createEventBookIndexes = (req, res) => {
//   const evId = req.params.id;
//   res.json({ success: true, msg: 'indexing...' });
//   let pages = [];
//   Event.getEvent(evId, (err, event) => {
//     if (err) {
//       console.log(err);
//       return;
//     }
//     console.log('event logged');
//     const members = event.sortedMembersForBook;
//     // newMem.getMemInEvent(evId, (err, members) => {
//     //   if (err) {
//     //     console.log(err);
//     //     return;
//     //   }
//     //   console.log('mems logged');
//     Plan.getPlansByEvent(evId, (err, plans) => {
//       if (err) {
//         console.log(err);
//         return;
//       }
//       console.log('plans logged');
//       let i = 0;
//       let planCount = 0;
//       let reverse = false;
//       let memCount = 0;
//       let NonPlanIds = [];
//       while (i < members.length) {
//         // console.log('mem: ' + i);
//         let page = {
//           pageWeight: 0,
//           pageProfiles: []
//         }
//         let weightCheck = true;
//         while (weightCheck) {
//           // console.log('pw: ' + page.pageWeight);
//           // console.log(members.length);
//           if (members[i]) {
//             let memFound = false;
//             for (const mem of event.members) {
//               // console.log(mem.member_id, members[i].member_id);
//               if (mem.member_id == members[i]._id) {
//                 memFound = true;
//                 NonPlanIds.push(members[i].member_id);
//                 let planFound = false;
//                 for (const pl of plans) {
//                   if (pl._id == mem.plan_id) {
//                     planFound = true;
//                     let tempWeight = page.pageWeight;
//                     let weight = 1;
//                     if (pl.size == '1') {
//                       tempWeight += 4;
//                       weight = 4;
//                     } else if (pl.size == '1/2') {
//                       tempWeight += 2
//                       weight = 2
//                     } else if (pl.size == '1/4') {
//                       tempWeight += 1
//                       weight = 1
//                     }
//                     // page.pageWeight = tempWeight;
//                     // page.pageProfiles.push({
//                     //   weight: weight,
//                     //   pageProfile: members[i].member_id
//                     // });
//                     if (tempWeight <= 4) {
//                       page.pageWeight = tempWeight;
//                       page.pageProfiles.push({
//                         weight: weight,
//                         pageProfile: members[i].member_id
//                       });
//                       if (tempWeight == 4) {
//                         weightCheck = false;
//                       }
//                     } else {
//                       weightCheck = false;
//                     }
//                   }
//                 }
//                 if (!planFound) {
//                   console.log('plan not found');
//                   page.pageWeight = page.pageWeight + 1;
//                   page.pageProfiles.push({
//                     weight: 1,
//                     pageProfile: members[i].member_id
//                   });
//                 }
//               }
//             }
//             if (!memFound) {
//               memCount++;
//               if (page.pageWeight + 1 <= 4) {
//                 page.pageWeight = page.pageWeight + 1;
//                 page.pageProfiles.push({
//                   weight: 1,
//                   pageProfile: members[i].member_id
//                 });
//               }
//             }
//           } else {
//             break;
//           }
//           i++;
//         }
//         // NonPlanIds = _.uniq(NonPlanIds);
//         // fs.writeFile('noplanids.txt', NonPlanIds, (err) => {
//         //   if (err) throw err;
//         //   console.log('logged');
//         // })
//         // console.log(NonPlanIds);
//         // console.log('mem not found: ' + memCount);
//         pages.push(page);
//         // console.log(page.pageProfiles.length);
//         // console.log(i);
//       }
//       // pages = JSON.stringify(pages);
//       // fs.writeFile(evId + '.json', pages);
//       Event.updateBook(evId, pages, (err, data) => {
//         if (err) {
//           console.log(err);
//         }
//         // console.log(data);
//         console.log('Pages Generated');
//         console.log(pages.length);
//         let total = 0;
//         for (const page of pages) {
//           total += page.pageProfiles.length;
//         }
//         console.log(total);
//         console.log(members.length);
//         console.log(members.length - total);
//       });
//     })
//     // })
//   })
// }

module.exports.delete = (req, res) => {
  const id = req.params._id;
  Event.deleteEvent(id, (err, data) => {
    if (err) {
      rc.setResponse(res, {
        msg: err.message,
        data: err
      });
    } else {
      if (data) {
        rc.setResponse(res, {
          success: true,
          msg: "Data deleted successfully",
          data: data
        });
      } else {
        rc.setResponse(res, {
          msg: "No data found by this id "
        });
      }
    }
  });
};



module.exports.subscribeMembersForWeb = (req, res, members, event, plan) => {
  // const members = req.body.members;
  // const event = req.body.event;
  // const plan = req.body.plan;
  Event.getEvent(event, (err, fetchedEvent) => {
    if (err) {
      rc.setResponse(res, {
        msg: "Error in fetching event"
      });
    } else {
      Plan.getPlan(plan, (err, fetchedPlan) => {
        if (err) {
          rc.setResponse(res, {
            msg: "Error in fetching plan"
          });
        } else {
          for (const member of members) {
            newMem.getMember(member, (err, data) => {
              if (err) {
                rc.setResponse(res, {
                  msg: "Error in fetching member",
                  data: err
                });
              } else {
                data.subscribed_events.push(event);
                let oldWallet = data.wallet;
                newWallet = oldWallet - fetchedPlan.plan_price;
                data.wallet = newWallet;
                let transactionToAdd = {
                  payee: data.member_id + '@' + data._id,
                  receiver: 0,
                  from: 'member',
                  to: 'company',
                  action: 'debit',
                  amount: fetchedPlan.plan_price,
                  prevBal: parseInt(data.wallet) + parseInt(fetchedPlan.plan_price),
                  Remark: data.member_name + '@' + data.member_mobile + ' ' + 'A: Member Subscription',
                  Refrence: fetchedPlan.name + '@' + fetchedPlan._id,
                  Date: Date.now(),
                  created_at: Date.now(),
                  created_by: req.user ? req.user.username : 'organic'
                }
                let transactionToAddCredit = {
                  payee: 0,
                  receiver: data.member_id + '@' + data._id,
                  from: 'CCAvenue',
                  to: 'member',
                  action: 'credit',
                  amount: fetchedPlan.plan_price,
                  prevBal: data.wallet,
                  Remark: data.member_name + '@' + data.member_mobile + ' ' + 'A: Member Wallet Recharge',
                  Refrence: fetchedPlan.name + '@' + fetchedPlan._id,
                  Date: Date.now(),
                  created_at: Date.now(),
                  created_by: req.user ? req.user.username : 'organic'
                }
                Transaction.newTransaction(transactionToAddCredit, (transdataCredit) => {
                  // console.log(transdata);
                  // return true;
                  Transaction.newTransaction(transactionToAdd, (transdata) => {
                    // console.log(transdata);
                    return true;
                  });
                });
                // let newTransaction = new Transaction({
                //   member_id: data,
                //   amount: fetchedPlan.plan_price,
                //   transaction: "Debit",
                //   remark: `${fetchedPlan.plan_price} debited for event ${fetchedEvent.Title} from ${data.member_name}'s wallet `,
                //   event_id: event,
                //   plan_id: plan
                // });
                // Transaction.addTransaction(newTransaction, (err, trans) => {
                //   if (err) {
                //     rc.setResponse(res, {
                //       msg: "Error in making transaction",
                //       data: err
                //     });
                //   } else {
                newMem.addNewMember(data, (err, updatedData) => {
                  if (err) {
                    rc.setResponse(res, {
                      msg: "Error in updating member",
                      data: err
                    });
                  } else {
                    let newMemPlan = {
                      member_id: member,
                      plan_id: plan
                    };
                    fetchedEvent.members.push(newMemPlan);
                    Event.addEvent(fetchedEvent, (err, newData) => {
                      if (err) {
                        // rc.setResponse(res,{
                        //   msg: 'Error in adding member and plan in event',
                        //   data: err
                        // })
                      } else {
                        rc.setResponse(res, {
                          success: true,
                          msg:
                            "Event added in all members and members and plan added in selected event"
                        });
                      }
                    });
                  }
                });
                // }
                // });
              }
            });
          }
        }
      });
    }
  });
};