const siteInfo = require('./../models/siteInfo');
const rc = require('./../controllers/responseController');

module.exports.addSiteInfo = async (req, res) => {
    let newSiteInfo = new siteInfo({
        metaDescription: req.body.metaDescription,
        siteLogo: req.body.siteLogo,
        watermarkLogo: req.body.watermarkLogo,
        siteIcon: req.body.siteIcon,
        contactNumber: req.body.contactNumber,
        contactEmail: req.body.contactEmail,
        createdBy: req.body.createdBy,
    });

    await siteInfo.addSiteInfo(newSiteInfo, (err, data) => {
        if (err) {
            rc.setResponse(
                res,
                {
                    msg: err.message,
                    data: err
                }
            );
        } else {
            rc.setResponse(
                res,
                {
                    success: true,
                    msg: 'created successfully',
                    data: data
                }
            );
        }
    });
};

module.exports.delete = (req, res) => {
    siteInfo.deleteSiteInfo(req.body.id, (err, data) => {
        if (err) {
            rc.setResponse(
                res,
                {
                    msg: err.message,
                    data: err
                }
            );
        } else {
            if (data) {
                rc.setResponse(
                    res,
                    {
                        success: true,
                        msg: 'deleted successfully',
                        data: data
                    }
                );
            } else {
                rc.setResponse(
                    res,
                    {
                        msg: 'No data found with this id'
                    }
                );
            }
        }
    });
}

module.exports.getDetails = (req, res) => {
    siteInfo.getSiteInfo((err, data) => {
        if (err) {
            rc.setResponse(
                res,
                {
                    msg: err.message,
                    data: err
                }
            );
        } else {
            if (data) {
                rc.setResponse(
                    res,
                    {
                        success: true,
                        msg: 'fetched successfully',
                        data: data
                    }
                );
            } else {
                this.addSiteInfo(req, res);
            }
        }
    });
}

module.exports.updateDetails = (req, res) => {
    siteInfo.getSiteInfo((err, data) => {
        if (err) {
            rc.setResponse(
                res,
                {
                    msg: err.message,
                    data: err
                }
            );
        } else {
            if (data) {
                let isChange = false;
                data.lastChanges = {};
                if (req.body.siteTitle) {
                    isChange = true;
                    data.lastChanges.siteTitle = data.siteTitle;
                    data.siteTitle = req.body.siteTitle
                }
                if (req.body.metaDescription) {
                    isChange = true;
                    data.lastChanges.metaDescription = data.metaDescription;
                    data.metaDescription = req.body.metaDescription
                }
                if (req.body.siteLogo) {
                    isChange = true;
                    data.lastChanges.siteLogo = data.siteLogo;
                    data.siteLogo = req.body.siteLogo
                }
                if (req.body.watermarkLogo) {
                    isChange = true;
                    data.lastChanges.watermarkLogo = data.watermarkLogo;
                    data.watermarkLogo = req.body.watermarkLogo
                }
                if (req.body.siteIcon) {
                    isChange = true;
                    data.lastChanges.siteIcon = data.siteIcon;
                    data.siteIcon = req.body.siteIcon
                }
                if (req.body.contactNumber) {
                    isChange = true;
                    data.lastChanges.contactNumber = data.contactNumber;
                    data.contactNumber = req.body.contactNumber
                }
                if (req.body.contactEmail) {
                    isChange = true;
                    data.lastChanges.contactEmail = data.contactEmail;
                    data.contactEmail = req.body.contactEmail
                }
                if (req.body.score) {
                    isChange = true;
                    data.lastChanges.score = data.score;
                    data.score = req.body.score
                }
                if (isChange) {
                    data.updatedAt = Date.now();
                    data.updatedBy = req.body.username;
                    siteInfo.addSiteInfo(data, (err, newData) => {
                        if (err) {
                            rc.setResponse(
                                res,
                                {
                                    msg: err.message,
                                    data: err
                                }
                            );
                        } else {
                            rc.setResponse(
                                res,
                                {
                                    success: true,
                                    msg: 'updated successfully',
                                    data: newData
                                }
                            );
                        }
                    });
                } else {
                    rc.setResponse(
                        res,
                        {
                            msg: 'No data to update'
                        }
                    );
                }
            } else {
                rc.setResponse(
                    res,
                    {
                        msg: 'No data found with this id'
                    }
                );
            }
        }
    });
}