const config = require('../config/database');
const mongoose = require('mongoose');

const ChatSchema = mongoose.Schema({
    msgType: {
        type: String
    },
    content: {
        type: String
    },
    msgBy: {
        type: String
    },
    msgFor: {
        type: String
    },
    approvedStatus: {
        type: String
    },
    readStatus: {
        type: String
    },
    isGroup: {
        type: String,
    },
    createdAt: {
        type: Date
    },
    updatedAt: {
        type: Date
    },
    updatedBy: {
        type: String
    },
    deleted: {
        type: Boolean,
        default: false
    }
}, { collection: 'lms_chats' });

const Chat = (module.exports = mongoose.model('lms_chats', ChatSchema));

// Create and update
module.exports.addChat = (newChat, callback) => {
    newChat.save(callback);
};

// get  
module.exports.getChatMsg = (id, callback) => {
    Chat.findById(id, callback);
};

// get
module.exports.getChatsByUser = (uname, fname, limit, skip, callback) => {
    Chat.find(
        {
            $or: [
                {
                    $and: [
                        { msgBy: uname },
                        { msgFor: fname }
                    ],
                },
                {
                    $and: [
                        { msgBy: fname },
                        { msgFor: uname }
                    ]
                }
            ],
            isGroup: 'N',
            deleted: false
        }
        , callback).sort({ createdAt: -1 }).skip(skip).limit(limit);
};

// get
module.exports.getUnreadChatsCount = (uname, callback) => {
    Chat.countDocuments(
        {
            msgFor: uname,
            readStatus: 'N',
            isGroup: 'N',
            deleted: false
        }
        , callback);
};
// get
module.exports.getUnreadChatsByUser = (uname, callback) => {
    Chat.find(
        {
            msgFor: uname,
            readStatus: 'N',
            isGroup: 'N',
            deleted: false
        }
        , callback);
};

// get
module.exports.readChatsByUser = (uname, fname, callback) => {
    Chat.updateMany(
        {
            $and: [
                { msgBy: fname },
                { msgFor: uname }
            ],
            isGroup: 'N',
            deleted: false
        }
        , {
            $set: {
                readStatus: 'Y'
            }
        }, callback);
};

//delete 
module.exports.deleteChat = (id, callback) => {
    Chat.findByIdAndUpdate(id, {
        $set: {
            deleted: true
        }
    });
};
