const mongoose = require('mongoose');
const config = require('../config/database');

const familyIncomeSchema = mongoose.Schema({
  family_income_id: { type: String, required: true },
  family_income_title: { type: String, required: true },
  family_income_status: { type: String },
  family_income_slug: { type: String },
  family_income_is_deleted: { type: Boolean, },
  family_income_added_by: { type: String },
  family_income_order: { type: String },
  family_income_created_on: { type: Date, default: Date.now() }
}, { collection: 'sbw_family_incomes' });

const FamilyIncome = (module.exports = mongoose.model('sbw_family_incomes', familyIncomeSchema));

// Add familyIncome
module.exports.addFamilyIncome = (familyIncome, callback) => {
  FamilyIncome.create(familyIncome, callback);
}

// read familyIncome
module.exports.getFamilyIncome = (callback) => {
  FamilyIncome.find(callback);
}
module.exports.getDesiredFamilyIncome = (id, callback) => {
  FamilyIncome.find({ family_income_id: { $in: id } }, callback);
}
// read familyIncome names
module.exports.getFamilyIncomeNames = (callback) => {
  FamilyIncome.find(null, { "family_income_title": true, family_income_id: true }, callback);
}

//find a familyIncome by id
module.exports.getFamilyIncomeById = (Id, callback) => {
  FamilyIncome.findOne({ "family_income_id": Id }, callback);
}

//find a familyIncome by title
module.exports.getFamilyIncomeByTitle = (title, callback) => {
  FamilyIncome.findOne({ "family_income_title": title }, callback);
}


// Update familyIncome
module.exports.updateFamilyIncome = (id, familyIncome, options, callback) => {
  var query = { id: id };
  var update = {};
  if (familyIncome.family_income_title) {
    update.family_income_title = familyIncome.family_income_title
  }
  if (familyIncome.family_income_status) {
    update.family_income_status = familyIncome.family_income_status
  }
  if (familyIncome.family_income_slug) {
    update.family_income_slug = familyIncome.family_income_slug
  }
  if (familyIncome.family_income_is_deleted) {
    update.family_income_is_deleted = familyIncome.family_income_is_deleted
  }
  if (familyIncome.family_income_added_by) {
    update.family_income_added_by = familyIncome.family_income_added_by
  }
  if (familyIncome.family_income_order) {
    update.family_income_order = familyIncome.family_income_order
  }

  FamilyIncome.findByIdAndUpdate(id, { $set: update }, callback);
}


// Delete FamilyIncome
module.exports.removeFamilyIncome = (id, callback) => {
  var query = { family_income_id: id };
  FamilyIncome.remove(query, callback);
}


