const mongoose = require("mongoose");
const config = require("../config/database");

const NewLeadSchema = mongoose.Schema({
  leadNumber: { type: Number, index: true },
  purpose: { type: String, default: "" },
  remark: { type: String, default: "" },
  source: { type: String, default: "" },
  DOB: { type: Date },
  name: { type: String, default: "" },
  callerId: { type: String },
  city: { type: String, default: "" },
  cityType: { type: String, default: "" },
  pcity: { type: Array, default: [] },
  state: { type: String },
  lastUpdate: {
    type: JSON,
    default: {}
  },
  country: { type: String },
  followup: { type: Array, default: [] },
  messages: { type: Array, default: [] },
  email: { type: String },
  gender: { type: String },
  mobile: { type: String, default: "" },
  wanumber: { type: String },
  sendAddress: { type: Array, default: [] },
  counsellor: { type: JSON },
  package: { type: String, default: "" },
  unit: { type: String, default: "" },
  rate: { type: Number, default: "" },
  discount: { type: Number, default: "" },
  quantity: { type: Number, default: "" },
  additionalCharges: { type: Number, default: "" },
  packageType: String,
  renewDate: Date,
  serviceCount: String,
  serviceGap: String,
  billingCount: String,
  billingGap: String,
  grossCharges: { type: Number, default: "" },
  totalCharges: { type: Number, default: "" },
  billing: { type: String, default: "" },
  shipping: { type: String, default: "" },
  allotedTo: { type: String, default: "none" },
  occupation: { type: String },
  rpay: { type: String },
  privilegeDiscount: { type: String },
  coupenSeries: { type: String },
  coupenNumber: { type: String },
  coupenDiscount: { type: Number },
  leadStatus: { type: String, default: "" },
  createdBy: { type: String, default: "" },
  mappedBy: { type: String, default: "" },
  member_id: { type: String },
  leadCompanyName: { type: String },
  billingName: { type: String },
  vip: {
    type: Boolean,
    default: false
  },
  poapproval: {
    type: Boolean,
    default: false
  },
  outstanding: {
    type: Boolean,
    default: false
  },
  createdAt: {
    type: Date,
    default: Date.now(),
    index: true
  },
  followupAt: {
    type: Date,
    index: true
  },
  updatedAt: {
    type: Date,
    default: Date.now(),
    index: true
  },
  transactionData: String,
  updatedBy: { type: String }
});

NewLeadSchema.index({ _id: 1, updatedAt: 1, createdAt: 1, followupAt: 1 });

const Lead = (module.exports = mongoose.model("lead", NewLeadSchema));

Lead.createIndexes();

module.exports.addNewLead = (newLead, callback) => {
  newLead.save(callback);
};

module.exports.getLeadById = (id, callback) => {
  Lead.findById(id, callback);
};

module.exports.getLeadByMobile = (mobile, callback) => {
  Lead.findOne({ mobile: mobile }, callback);
};

module.exports.getLeadByMobiles = (mobile, callback) => {
  Lead.find({ mobile: mobile }, callback);
};

module.exports.deleteLeadById = (id, callback) => {
  Lead.findByIdAndDelete(id, callback);
};

module.exports.getNewLeads = callback => {
  Lead.find({ $or: [{ followup: { $eq: [] } }, { "followup.status": "MAP" }] }, callback).sort("-updatedAt");
};

module.exports.getOldLeads = callback => {
  Lead.find({ followup: { $ne: [] } }, callback).sort("-updatedAt");
};

module.exports.getLeads = callback => {
  Lead.find(callback).sort("-updatedAt");
};

module.exports.getUnallotedLeads = (query, callback) => {
  Lead.find({ $and: query }, callback).sort("-updatedAt");
};

module.exports.getLimitedUnallotedLeads = (query, limit, skip, callback) => {
  Lead.find({ $and: query }, callback).limit(limit).skip(skip);
  // .sort("-updatedAt");
};

module.exports.getAllLeadsCount = callback => {
  // Lead.find({ allotedTo: 'none' }, callback);
  Lead.countDocuments(callback);
};

module.exports.getAllotedLeadsCount = callback => {
  // Lead.find({ allotedTo: 'none' }, callback);
  Lead.countDocuments({ allotedTo: { $ne: "none" } }, callback);
};

// module.exports.getUnallotedLeadsCount = callback => {
//   // Lead.find({ allotedTo: 'none' }, callback);
//   Lead.countDocuments({ allotedTo: "none" }, callback);
// };

module.exports.getUnallotedLeadsCount = (query, callback) => {
  // Lead.find({ allotedTo: 'none' }, callback);
  Lead.countDocuments({ $and: query }, callback);
};

module.exports.getAllotedLeads = callback => {
  Lead.find({ allotedTo: { $ne: "none" } }, callback).sort("-updatedAt");
};

module.exports.getNewLeadsByUser = (username, callback) => {
  Lead.find({ allotedTo: username, $or: [{ followup: { $eq: [] } }, { "followup.status": "MAP" }] }, callback).sort(
    "-updatedAt"
  );
};

module.exports.getOldLeadsByUser = (username, callback) => {
  Lead.find({ allotedTo: username, followup: { $ne: [] } }, callback).sort(
    "-updatedAt"
  );
};

module.exports.getLeadsByUser = (username, callback) => {
  Lead.find({ allotedTo: username }, callback).sort("-updatedAt");
};

module.exports.getNewLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, $or: [{ followup: { $eq: [] } }, { "followup.status": "MAP" }] },
    callback
  ).sort("-updatedAt");
};

module.exports.getOldLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, followup: { $ne: [] } },
    callback
  ).sort("-updatedAt");
};

module.exports.getCallbackLeadsByUser = callback => {
  Lead.find({ "followup.status": "CALLBACK" }, callback).sort("-updatedAt");
};

module.exports.getNpcLeadsByUser = callback => {
  Lead.find({ "followup.status": "NPC" }, callback).sort("-updatedAt");
};

module.exports.getApprovedLeadsByUser = callback => {
  Lead.find(
    { "followup.status": { $in: ["APPROVED", "PAID", "PAYMENT RECD"] } },
    callback
  ).sort("-updatedAt");
};

module.exports.getInterestedLeadsByUser = callback => {
  Lead.find({ "followup.status": "INTERESTED" }, callback).sort("-updatedAt");
};

module.exports.getLeadsStatus = (status, callback) => {
  Lead.find({ "followup.status": status }, callback).sort("-updatedAt");
};

module.exports.getLeadsStatusByUser = (status, username, callback) => {
  Lead.find({ allotedTo: username, "followup.status": status }, callback).sort("-updatedAt");
};

module.exports.getLeadsStatusByUserByUsernames = (status, usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": status },
    callback
  ).sort("-updatedAt");
};

module.exports.getLeadsStatuses = (statuses, callback) => {
  Lead.find({ "followup.status": { $in: statuses } }, callback).sort("-updatedAt");
};

module.exports.getLeadsStatusesByUser = (statuses, username, callback) => {
  Lead.find({ allotedTo: username, "followup.status": { $in: statuses } }, callback).sort("-updatedAt");
};

module.exports.getLeadsStatusesByUserByUsernames = (statuses, usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": { $in: statuses } },
    callback
  ).sort("-updatedAt");
};

module.exports.getProspectLeadsByUser = callback => {
  Lead.find({ "followup.status": { $in: ["PROSPECT / QUOTE", "PROSPECT"] } }, callback).sort("-updatedAt");
};

module.exports.getVipByUser = callback => {
  Lead.find({ vip: true }, callback).sort("-updatedAt");
};

module.exports.getPoaByUser = callback => {
  Lead.find({ poapproval: true }, callback).sort("-updatedAt");
};

module.exports.getClosedLeadsByUser = callback => {
  Lead.find({ "followup.status": "CLOSED" }, callback).sort("-updatedAt");
};

module.exports.getCallbackLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": "CALLBACK" },
    callback
  ).sort("-updatedAt");
};

module.exports.getNpcLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": "NPC" },
    callback
  ).sort("-updatedAt");
};

module.exports.getApprovedLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    {
      allotedTo: { $in: usernames },
      "followup.status": { $in: ["APPROVED", "PAID", "PAYMENT RECD"] }
    },
    callback
  ).sort("-updatedAt");
};

module.exports.getInterestedLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": "INTERESTED" },
    callback
  ).sort("-updatedAt");
};

module.exports.getProspectLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": { $in: ["PROSPECT / QUOTE", "PROSPECT"] } },
    callback
  ).sort("-updatedAt");
};

module.exports.getVipByUserByUsernames = (usernames, callback) => {
  Lead.find({ allotedTo: { $in: usernames }, vip: true }, callback).sort(
    "-updatedAt"
  );
};

module.exports.getPoaByUserByUsernames = (usernames, callback) => {
  Lead.find({ allotedTo: { $in: usernames }, poapproval: true }, callback).sort(
    "-updatedAt"
  );
};

module.exports.getClosedLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, "followup.status": "CLOSED" },
    callback
  ).sort("-updatedAt");
};

module.exports.getLeadsByUserByUsernames = (usernames, callback) => {
  Lead.find(
    { allotedTo: { $in: usernames }, followup: { $ne: [] } },
    callback
  ).sort("-updatedAt");
};

module.exports.getLeadsReport = (data, callback) => {
  Lead.find(
    {
      allotedTo: { $in: data.users },
      $and: [
        { createdAt: { $gte: data.fromDate } },
        { createdAt: { $lte: data.toDate } }
      ]
    },
    callback
  ).sort("-updatedAt");
};

// testing inserting array using insert command
module.exports.insertArrayofLeads = (leads, callback) => {
  for (const ld of leads) {
    ld.city = ld.city.split('$');
    ld.city = ld.city.join(',');
    ld.createdAt = Date.now()
  }
  Lead.insertMany(leads, callback);
};

module.exports.getDuplicateLeadByMobile = callback => {
  Lead.aggregate(
    [
      {
        $group: { _id: "$mobile", count: { $sum: 1 } }
      },
      { $match: { _id: { $ne: null }, count: { $gt: 1 } } }
    ],
    callback
  );
};

module.exports.getLeadUsernames = callback => {
  Lead.aggregate(
    [
      {
        $group: { _id: "$allotedTo", count: { $sum: 1 } }
      },
    ],
    callback
  );
};

module.exports.countLeadsByLastFollowUpStatus = (statuses, callback) => {
  return Lead.aggregate([
    {
      $addFields: {
        "lastStatus": { $arrayElemAt: ["$followup.status", -1] }
      }
    },
    {
      $match: {
        "lastStatus": { $in: statuses }
      }
    },
    {
      $count: 'lastStatusCount'
    }
  ], callback);
}

module.exports.getLeadsByLastFollowUpStatus = (statuses, callback) => {
  return Lead.aggregate([
    {
      $addFields: {
        "lastStatus": { $arrayElemAt: ["$followup.status", -1] }
      }
    },
    {
      $match: {
        "lastStatus": { $in: statuses }
      }
    },
    {
      $addFields: {
        "stringId": {
          $toString: "$_id"
        }
      }
    },
    {
      $addFields: {
        "stringLeadNumber": {
          $toString: "$leadNumber"
        }
      }
    },
    {
      $lookup: {
        from: 'leadservices',
        localField: 'stringId',
        foreignField: 'leadId',
        as: 'leadServices'
      }
    },
    {
      $addFields: {
        "services": {
          $arrayElemAt: ["$leadServices.servicesData", 0]
        }
      }
    },
    {
      $lookup: {
        from: 'leadbillings',
        localField: 'stringId',
        foreignField: 'leadId',
        as: 'leadBillings'
      }
    },
    {
      $addFields: {
        "billings": {
          $arrayElemAt: ["$leadBillings.billingsData", 0]
        }
      }
    },
    {
      $lookup: {
        from: 'leadtransactions',
        localField: 'stringLeadNumber',
        foreignField: 'leadNumber',
        as: 'transactions'
      }
    },
    {
      $project: {
        "lastUpdate": 0
      }
    }
  ], callback);
}

module.exports.getLeadsByLastFollowUpStatusByUsernames = (statuses, usernames, callback) => {
  return Lead.aggregate([
    {
      $addFields: {
        "lastStatus": { $arrayElemAt: ["$followup.status", -1] }
      }
    },
    {
      $match: {
        "lastStatus": { $in: statuses },
        "allotedTo": { $in: usernames }
      }
    },
    {
      $addFields: {
        "stringId": {
          $toString: "$_id"
        }
      }
    },
    {
      $addFields: {
        "stringLeadNumber": {
          $toString: "$leadNumber"
        }
      }
    },
    {
      $lookup: {
        from: 'leadservices',
        localField: 'stringId',
        foreignField: 'leadId',
        as: 'leadServices'
      }
    },
    {
      $addFields: {
        "services": {
          $arrayElemAt: ["$leadServices.servicesData", 0]
        }
      }
    },
    {
      $lookup: {
        from: 'leadbillings',
        localField: 'stringId',
        foreignField: 'leadId',
        as: 'leadBillings'
      }
    },
    {
      $addFields: {
        "billings": {
          $arrayElemAt: ["$leadBillings.billingsData", 0]
        }
      }
    },
    {
      $lookup: {
        from: 'leadtransactions',
        localField: 'stringLeadNumber',
        foreignField: 'leadNumber',
        as: 'transactions'
      }
    },
    {
      $project: {
        "lastUpdate": 0
      }
    }
  ], callback);
}
