const config = require('../config/database');
const mongoose = require('mongoose');

const ReligionMasterSchema = mongoose.Schema({
    Religion: {
        type: String
    },
    Caste: {
        type: String
    },
    SubCaste: {
        type: String
    },
    Gotra: {
        type: String
    },

}, { collection: 'ss_new_religion_masters' });

const ReligionMaster = (module.exports = mongoose.model('ss_new_religion_masters', ReligionMasterSchema));


//get distinct religions
module.exports.getAllReligions = (callback) => {
    ReligionMaster.aggregate(
        [
            {
                $group: { _id: "$Religion" }
            }
        ], callback
    )
}
//Get all castes

module.exports.getAllDistinctCastes = (callback) => {
    ReligionMaster.aggregate(
        [
            {
                $match: { "Caste": { $ne: null } }
            },
            {
                $group: { _id: "$Caste" }
            }

        ], callback
    )
}
// get all subCastes 
module.exports.getAllDistinctSubCastes = (callback) => {
    ReligionMaster.aggregate(
        [
            {
                $match: { "SubCaste": { $ne: null } }
            },

            {
                $group: { _id: "$SubCaste" }
            }

        ], callback
    )
}
// get gotra 
module.exports.getAllDistinctGotras = (callback) => {
    ReligionMaster.aggregate(
        [{
            $match: { "Gotra": { $ne: null } }
        },

        {
            $group: { _id: "$Gotra" }
        }

        ], callback
    )
}
// get castes by religion name
module.exports.getAllCastes = (religion, callback) => {
    ReligionMaster.aggregate(
        [
            {
                $match: { "Caste": { $ne: null } }
            },
            {
                $match: { "Religion": religion }
            },
            {
                $group: { _id: "$Caste" }
            }

        ], callback
    )
}
// get subCastes by religion,caste name
module.exports.getAllSubCastes = (religion, caste, callback) => {
    ReligionMaster.aggregate(
        [
            {
                $match: { "SubCaste": { $ne: null } }
            },
            {
                $match: { "Religion": religion }
            },
            {
                $match: { "Caste": caste }
            },
            {
                $group: { _id: "$SubCaste" }
            }

        ], callback
    )
}
// get gotra by religion,caste,subcaste name
module.exports.getAllGotras = (religion, caste, subCaste, callback) => {
    ReligionMaster.aggregate(
        [{
            $match: { "Gotra": { $ne: null } }
        },
        {
            $match: { "Religion": religion }
        },
        {
            $match: { "Caste": caste }
        },
        {
            $match: { "SubCaste": subCaste }
        },
        {
            $group: { _id: "$Gotra" }
        }

        ], callback
    )
}
//Create Religion
module.exports.createReligion = (religion, callback) => {
    religion.save(callback);
}
//Update Religion
module.exports.updateReligion = (religion, newReligion, callback) => {
    ReligionMaster.updateMany({ "Religion": religion }, { $set: { "Religion": newReligion } }, callback);
}
//Delete religion
module.exports.deleteReligion = (religion, callback) => {
    ReligionMaster.deleteMany({ "Religion": religion }, callback);
}
//Create Caste
module.exports.createCaste = (caste, callback) => {
    caste.save(callback);
}
//Update Caste
module.exports.updateCaste = (religion, caste, newCaste, callback) => {
    ReligionMaster.updateMany({ "Religion": religion, "Caste": caste }, { $set: { "Caste": newCaste } }, callback);
}
//Delete caste
module.exports.deleteCaste = (religion, caste, callback) => {
    ReligionMaster.deleteMany({ "Religion": religion, "Caste": caste }, callback);
}

//Create SubCaste
module.exports.createSubCaste = (subCaste, callback) => {
    subCaste.save(callback);
}
//Update SubCaste
module.exports.updateSubCaste = (religion, caste, subCaste, newSubCaste, callback) => {
    ReligionMaster.updateMany({ "Religion": religion, "Caste": caste, "SubCaste": subCaste }, { $set: { "SubCaste": newSubCaste } }, callback);
}
//Delete SubCaste
module.exports.deleteSubCaste = (religion, caste, subCaste, callback) => {
    ReligionMaster.deleteMany({ "Religion": religion, "Caste": caste, "SubCaste": subCaste }, callback);
}
//Create Gotra
module.exports.createGotra = (gotra, callback) => {
    gotra.save(callback);
}
//Update Gotra
module.exports.updateGotra = (religion, caste, subCaste, gotra, newGotra, callback) => {
    ReligionMaster.updateMany({ "Religion": religion, "Caste": caste, "SubCaste": subCaste, "Gotra": gotra }, { $set: { "Gotra": newGotra } }, callback);
}
//Delete Gotra
module.exports.deleteGotra = (religion, caste, subCaste, gotra, callback) => {
    ReligionMaster.deleteMany({ "Religion": religion, "Caste": caste, "SubCaste": subCaste, "Gotra": gotra }, callback);
}


//delete 
module.exports.deleteReligionMaster = (id, callback) => {
    ReligionMaster.findByIdAndDelete(id, callback);
};


// Create and update
module.exports.addReligionMaster = (newMaster, callback) => {
    newMaster.save(callback);

};
module.exports.getReligionMaster = (id, callback) => {
    ReligionMaster.findOne({ _id: id }, callback);
}
// get all 
module.exports.getAllReligionMasters = (callback) => {
    ReligionMaster.find(callback);
};