const mongoose = require('mongoose');
const config = require('../config/database');

const NewOfficeSchema = mongoose.Schema({
  // office details
  officeName: { type: String },
  officeLocation: { type: Array, default: [] },
  officeLocationType: { type: String },
  officeAddress: { type: String },
  officeContact: { type: String },
  officeCity: { type: JSON },
  officeType: { type: String },
  officePackage: { type: String, default: 'main' },
  officeWallet: { type: String },
  officePercentage: { type: String },
  officeAdmin: { type: String },
  topOffice: { type: String },
  createdBy: { type: String },
  createdAt: { type: Date, default: Date.now() },
  updatedBy: { type: String },
  updatedAt: { type: Date }
});

const Office = (module.exports = mongoose.model('office', NewOfficeSchema));

module.exports.addOffice = (newOffice, callback) => {
  newOffice.save(callback);
};

module.exports.getOfficeById = (id, callback) => {
  Office.findById(id, callback);
};

module.exports.getOfficeByName = (officeName, callback) => {
  Office.findOne({ officeName: officeName }, callback);
};

module.exports.getOfficesByPackage = (officePackage, callback) => {
  Office.find({ officePackage: officePackage }, callback);
};

module.exports.getOfficesByTopOffice = (topOffice, callback) => {
  Office.find({ topOffice: topOffice }, callback);
};

module.exports.getOfficesByTopOfficeAndPackage = (topOffice, officePackage, callback) => {
  Office.find({$and: [{topOffice: topOffice}, { officePackage: officePackage }]}, callback);
};

module.exports.getAllOffice = callback => {
  Office.find(callback);
};

module.exports.deleteOffice = (name, callback) => {
  Office.findOneAndDelete({ officeName: name }, callback);
};


