const mongoose = require("mongoose");
const config = require("../config/database");

const DesiredSchema = mongoose.Schema(
  {
    desired_id: {
      type: String
    },
    desired_member_slug: {
      type: String,
      default: ''
    },
    desired_age_from: {
      type: String,
      default: ''
    },
    desired_age_to: {
      type: String,
      default: ''
    },
    desired_gender: {
      type: String,
      default: ''
    },
    desired_body_type: {
      type: Array,
      default: []
    },
    desired_height_from: {
      type: String,
      default: ''
    },
    desired_height_to: {
      type: String,
      default: ''
    },
    desired_weight_from: {
      type: String,
      default: ''
    },
    desired_weight_to: {
      type: String,
      default: ''
    },
    desired_maritial_status: {
      type: String,
      default: ''
    },
    desired_country: {
      type: Array,
      default: []
    },
    desired_religion: {
      type: Array,
      default: []
    },
    desired_religion_all: {
      type: String,
      default: 'N'
    },
    desired_caste: {
      type: Array,
      default: []
    },
    desired_mother_tongue: {
      type: Array,
      default: []
    },
    desired_manglik: {
      type: String,
      default: ''
    },
    // desired_gotra: {
    //     type: Array
    // },
    // desired_mama_gotra: {
    //     type: Array
    // },
    desired_education: {
      type: Array,
      default: []
    },
    desired_occupation: {
      type: Array,
      default: []
    },
    desired_income: {
      type: Array,
      default: []
    },
    desired_own_house: {
      type: String,
      default: ''
    },
    desired_family_status: {
      type: Array,
      default: []
    },
    desired_family_income: {
      type: Array,
      default: []
    },
    desired_family_type: {
      type: String,
      default: ''
    },
    desired_family_value: {
      type: Array,
      default: []
    },
    desired_family_residential_status: {
      type: Array,
      default: []
    },
    desired_eating_habit: {
      type: String,
      default: ''
    },
    desired_drinking_habit: {
      type: String,
      default: ''
    },
    desired_smoking_habit: {
      type: String,
      default: ''
    },
    desired_complexion: {
      type: String,
      default: ''
    },
    desired_abled: {
      type: Array,
      default: []
    },
    desired_abled_details: {
      type: String,
      default: ''
    },
    desired_about_partner: {
      type: String,
      default: ''
    },
    desired_is_deleted: {
      type: String,
      default: ''
    },
    desired_created_on: {
      type: Date
    },
    desired_updated_on: {
      type: Date
    },
    desired_member_id: {
      type: String
    },
    desired_user_id: {
      type: String,
      default: ''
    },
    desired_status: {
      type: String,
      default: ''
    },
    isOld: {
      type: String,
      default: "N"
    }
  },
  { collection: "sbw_desired" }
);

DesiredSchema.index({ _id: 1, desired_member_id: 1 });

const Desired = module.exports = mongoose.model("sbw_desired", DesiredSchema);

// Create and update
module.exports.addDesired = (newDesired, callback) => {
  newDesired.save(callback);
};

//get desired with projection 
module.exports.getDesiredWithProjection = (callback) => {
  Desired.aggregate([

    {
      $lookup: {
        from: "sbw_countries",
        localField: "desired_country",
        foreignField: "country_id",
        as: "countryx"
      }
    },
    {
      $lookup: {
        from: "sbw_religions",
        localField: "desired_religion",
        foreignField: "religion_id",
        as: "religionx"
      }
    },
    {
      $lookup: {
        from: "sbw_castes",
        localField: "desired_caste",
        foreignField: "caste_id",
        as: "castex"
      }
    },
    {
      $lookup: {
        from: "sbw_mother_tongues",
        localField: "desired_mother_tongue",
        foreignField: "mother_tongue_id",
        as: "mtx"
      }
    },
    {
      $lookup: {
        from: "sbw_education_details",
        localField: "desired_education",
        foreignField: "edu_detail_id",
        as: "eduDetailx"
      }
    },
    {
      $lookup: {
        from: "sbw_occupations",
        localField: "desired_occupation",
        foreignField: "occupation_id",
        as: "occupationx"
      }
    },
    {
      $lookup: {
        from: "sbw_family_incomes",
        localField: "desired_income",
        foreignField: "family_income_id",
        as: "incomex"
      }
    },
    {
      $lookup: {
        from: "sbw_family_status",
        localField: "desired_family_status",
        foreignField: "family_status_id",
        as: "familyStatusx"
      }
    },
    {
      $lookup: {
        from: "sbw_family_incomes",
        localField: "desired_family_income",
        foreignField: "family_income_id",
        as: "familyIncomex"
      }
    },
    {
      $lookup: {
        from: "sbw_family_values",
        localField: "desired_family_value",
        foreignField: "family_value_id",
        as: "familyValuex"
      }
    },
    {
      $unwind: { path: "$countryx", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$religionx", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$castex", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$mtx", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$eduDetailx", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$occupationx", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$incomex", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$familyStatusx", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$familyIncomex", preserveNullAndEmptyArrays: true }
    },
    {
      $unwind: { path: "$familyValuex", preserveNullAndEmptyArrays: true }
    },

    {
      $project: {


        desired_country: "$countryx.country_name",
        desired_religion: "$religionx.religion_name",
        desired_caste: "$castex.caste_name",
        desired_mother_tongue: "$mtx.mother_tongue_name",
        desired_education: "$eduDetailx.edu_detail_title",
        desired_occupation: "$occupationx.occupation_title",
        desired_income: "$incomex.family_income_title",
        desired_family_status: "$familyStatusx.family_status_title",
        desired_family_income: "$familyIncomex.family_income_title",
        desired_family_value: "$familyValuex.family_value_title",
        desired_id: 1,
        desired_member_slug: 1,
        desired_age_from: 1,
        desired_age_to: 1,
        desired_gender: 1,
        desired_body_type: 1,
        desired_height_from: 1,
        desired_height_to: 1,
        desired_weight_from: 1,
        desired_weight_to: 1,
        desired_maritial_status: 1,
        desired_manglik: 1,
        desired_own_house: 1,
        desired_family_type: 1,
        desired_family_residential_status: 1,
        desired_eating_habit: 1,
        desired_drinking_habit: 1,
        desired_smoking_habit: 1,
        desired_complexion: 1,
        desired_abled: 1,
        desired_abled_details: 1,
        desired_about_partner: 1,
        desired_is_deleted: 1,
        desired_created_on: 1,
        desired_member_id: 1,
        desired_user_id: 1,
        desired_status: 1,

      }
    }
  ], callback)
}
// Get by id
module.exports.getDesired = (id, callback) => {
  Desired.findById(id, callback);
};
// Get by member id
module.exports.getDesiredByMember = (id, callback) => {
  Desired.findOne({ desired_member_id: id }, callback);
};
//Get all desireds
module.exports.getAllDesired = callback => {
  console.log('Updating...');
  Desired.updateMany({}, { $set: { isOld: 'Y' } }, (err, res) => {
    if (err) throw err;
    if (res) {
      console.log('Updated');
    }
  });
  Desired.find(callback);
};


//Delete by id
module.exports.deleteDesired = (id, callback) => {
  Desired.findByIdAndDelete(id, callback);
};
