const mongoose = require('mongoose');
const config = require('../config/database');

const SSNotificationSchema = mongoose.Schema({
    notificationType: {
        type: String
    },
    notificationData: {
        type: JSON
    },
    title: {
        type: String
    },
    msg: {
        type: String
    },
    enable: {
        type: Boolean,
        default: true
    },
    read: {
        type: Boolean,
        default: false
    },
    createdBy: {
        type: String
    },
    createdAt: {
        type: Date
    },
    updatedAt: {
        type: Date
    }
});

const SSNotification = (module.exports = mongoose.model('SSNotification', SSNotificationSchema));

module.exports.addSSNotification = (newSSNotification, callback) => {
    newSSNotification.createdAt = Date.now();
    newSSNotification.save(callback);
};

module.exports.getSSNotificationsByMemberId = (id, member_id, callback) => {
    SSNotification.find({ _id: id, "notificationData.member_id": member_id, enable: true }, callback);
};

module.exports.getAllSSNotifications = (skip, callback) => {
    SSNotification.find({ enable: true }, callback).sort("-createdAt").skip(skip).limit(2000);
};

module.exports.getAllDisabledSSNotifications = (skip, callback) => {
    SSNotification.find({ enable: false }, callback).sort("-createdAt").skip(skip).limit(2000);
};

module.exports.disableNotification = (id, callback) => {
    SSNotification.findByIdAndUpdate(id, { $set: { enable: false } }, callback);
};

module.exports.searchInSSNotifications = (keyword, skip, callback) => {
    SSNotification.find({
        $or: [
            { notificationType: keyword },
            { title: keyword },
            { msg: keyword }
        ]
    }, callback).sort("-createdAt").skip(skip).limit(2000);
};
