const mongoose = require('mongoose');
const config = require('../config/database');

const ActivitySchema = mongoose.Schema({
    activityType: {
        type: String
    },
    activityData: {
        type: JSON
    },
    User: {
        type: JSON
    },
    UserId: {
        type: String
    },
    MemberId: {
        type: String
    },
    msg: {
        type: String
    },
    createdAt: {
        type: Date
    }
});

const Activity = (module.exports = mongoose.model('ssactivity', ActivitySchema));

module.exports.addActivity = (newActivity, callback) => {
    newActivity.createdAt = Date.now();
    newActivity.save(callback);
};

module.exports.getActivityById = (id, usernames, callback) => {
    Activity.findOne({ $and: [{ _id: id }, { "User.username": { $in: usernames } }] }, callback);
};

module.exports.getAllActivities = (skip, usernames, data, callback) => {
    // console.log(usernames);
    Activity.find({
        $and: [
            { "User.username": { $in: usernames } },
            { createdAt: { $gte: data.fromDate } },
            { createdAt: { $lte: data.toDate } }
        ]
    }, callback).sort("-createdAt").skip(skip);
};

module.exports.searchInActivities = (keyword, skip, usernames, data, callback) => {
    Activity.find({
        $and: [{
            $or: [
                { activityType: keyword },
                { msg: keyword },
                { "activityData.member.member_id": keyword },
                { "activityData.member.member_name": keyword },
                { "activityData.member.member_mobile": keyword },
                { "activityData.newMember.member_name": keyword },
                { "activityData.newMember.member_mobile": keyword },
                { "User.username": keyword },
            ]
        }, { "User.username": { $in: usernames } },
        { createdAt: { $gte: data.fromDate } },
        { createdAt: { $lte: data.toDate } }]
    }, callback).sort("-createdAt").skip(skip);
};

module.exports.searchInActivitiesByUser = (keyword, skip, usernames, data, callback) => {
    // console.log(keyword);
    // Activity.find({
    //     "User.username": keyword, "User.username": { $in: usernames },
    //     $and: [
    //         { createdAt: { $gte: data.fromDate } },
    //         { createdAt: { $lte: data.toDate } }
    //     ]
    // }, callback).sort("-createdAt").skip(skip);
    Activity.find({
        $and: [
            { "User.username": keyword },
            { "User.username": { $in: usernames } },
            { createdAt: { $gte: data.fromDate } },
            { createdAt: { $lte: data.toDate } }
        ]
    }, callback);
};

module.exports.searchInActivitiesByMemberID = (keyword, skip, usernames, data, callback) => {
    Activity.find({
        $and: [{
            $or: [
                { "activityData.member.member_id": keyword },
            ]
        }, { "User.username": { $in: usernames } },
        { createdAt: { $gte: data.fromDate } },
        { createdAt: { $lte: data.toDate } }]
    }, callback).sort("-createdAt").skip(skip);
};
