const mongoose = require('mongoose');
const config = require('../config/database');
const NewMember = require('./new_member');

const TransactionSchema = mongoose.Schema({
  payee: String,
  receiver: String,
  credit: String,
  debit: String,
  prevBal: String,
  newBal: String,
  from: String,
  to: String,
  action: String,
  amount: String,
  Remark: String,
  Refrence: String,
  Date: Date,
  isDeleted: {
    type: String,
    default: 'N'
  },
  created_at: {
    type: Date
  },
  created_by: String
});

const Transaction = (module.exports = mongoose.model(
  'Transaction',
  TransactionSchema
));

module.exports.addTransaction = (newTransaction, callback) => {
  console.log(newTransaction);
  newTransaction.Date = newTransaction.Date ? newTransaction.Date : Date.now();
  newTransaction.created_at = Date.now();
  newTransaction.save(callback);
};

module.exports.updateTransaction = (id, newTransaction, callback) => {
  // newTransaction.save(callback);
  Transaction.findByIdAndUpdate(id, newTransaction, callback);
};

module.exports.getTransactions = callback => {
  Transaction.find(callback);
};

module.exports.getTransaction = (id, callback) => {
  Transaction.findById(id, callback);
};

module.exports.getTransactionByFieldName = (fieldName, fieldValue, callback) => {
  let query = {};
  query[fieldName] = fieldValue;
  Transaction.find(query, callback);
};

module.exports.deleteTransactionById = (id, callback) => {
  Transaction.deleteOne({ _id: id }, callback);
};

module.exports.newTransaction = (transactionToAdd, callback) => {
  const newTransaction = new Transaction(transactionToAdd);
  console.log('Transaction initiated');
  if (!newTransaction) {
    console.log('No Transaction');
    callback({
      success: false,
      msg: 'No transaction to perform',
      data: null
    });
  }
  Transaction.addTransaction(newTransaction, (err, doc) => {
    if (err) {
      console.log('Transaction error while adding');
      callback({
        success: false,
        data: null,
        msg: err.message
      });
    } else {
      console.log('Transaction Added');
      let id;
      let transAmount;
      if (transactionToAdd.action === 'debit') {
        id = (transactionToAdd.payee).split('@')[1];
        transAmount = 0 - Math.abs(parseInt(transactionToAdd.amount));
      } else {
        id = (transactionToAdd.receiver).split('@')[1];
        transAmount = parseInt(transactionToAdd.amount);
      }
      // console.log(transAmount);
      NewMember.updateWallet(id, transAmount, doc._id, (m_err, m_doc) => {
        if (m_err) {
          console.log('Transaction Member Wallet Update Error');
          Transaction.deleteTransactionById(doc._id, (t_err, t_doc) => {
            if (t_err) {
              console.log('Transaction delete error');
              callback({ success: false, data: null, msg: t_err.message });
            } else {
              console.log('Transaction Deleted');
              callback({ success: false, data: null, msg: m_err });
            }
          });
        } else {
          console.log('Transaction Member Wallet Updated');
          NewMember.findById(id, (err, tmem) => {
            if (err) {
              console.log('Transaction Member By Id search error');
              console.log(err.message);
            } else {
              console.log('Transaction Member By Id found');
              let newRemark = tmem.member_name + '@' + tmem.member_mobile + ' ' + doc.Remark;
              Transaction.findByIdAndUpdate(doc._id, { $set: { Remark: newRemark } }, (err, updatedTrans) => {
                console.log('Transaction Remark Updated');
                console.log('err: ');
                console.log(err);
                console.log('data: ');
                console.log(updatedTrans);
              });
            }
          });
          callback({
            success: true,
            msg: 'Transaction Successfull',
            data: doc
          });
        }
      });
    }
  })
}