const _ = require('lodash');
const express = require('express');
const router = express.Router();
const passport = require('passport');
const User = require('../models/user');
const City = require('../models/city');
const State = require('../models/state');
const Helpers = require('../Helpers/Helpers');

router.get(
  '/get-cities',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    City.getCities((err, cities) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'Something went wrong', err: err });
      } else {
        res
          .status(200)
          .json({ success: true, msg: 'cities are found', cities: cities });
      }
    });
  }
);

router.post(
  '/get-cities',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let dist = req.body.district;
    City.getCitiesByDistrict(dist, (err, cities) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'Something went wrong', err: err });
      } else {
        res
          .status(200)
          .json({ success: true, msg: 'cities are found', cities: cities });
      }
    });
  }
);

router.post(
  '/create-city',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let name = Helpers.firstUpper(req.body.cityName);

    City.getCityByName(name, (err, city) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'Something went wrong', err: err });
      } else {
        if (city) {
          res.status(200).json({ success: false, msg: 'city already exist' });
        } else {
          let newCity = new City({
            view: req.body.viewOpt,
            name: Helpers.firstUpper(req.body.cityName),
            state: req.body.stateName,
            country: req.body.countryName,
            district: req.body.districtName,
          });
          City.addNewCity(newCity, (err, city) => {
            if (err) {
              res
                .status(200)
                .json({ success: false, msg: 'Error occured', err: err });
            } else {
              if (city) {
                res
                  .status(200)
                  .json({ success: true, msg: 'city added successfully ' });
              } else {
                res
                  .status(200)
                  .json({ success: false, msg: 'Something went wrong. ' });
              }
            }
          });
        }
      }
    });
  }
);

router.post(
  '/allot-city',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    State.getStatebyName(req.body.state, (err, state) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'error occured', err: err });
      } else {
        if (state) {
          state.cities.push(req.body.city);
          state.allotedBy = req.body.allotedBy;
          State.addState(state, (err, state) => {
            if (err) {
              res
                .status(200)
                .json({ success: false, msg: 'error occured', err: err });
            } else {
              res.status(200).json({
                success: true,
                msg: 'city and states are assigned',
                state: state
              });
            }
          });
        } else {
          res.status(200).json({ success: false, msg: 'state not found' });
        }
      }
    });
  }
);

router.post('/remove-city', (req, res) => {

});

// router.post(
//   '/remove-city',
//   passport.authenticate('jwt', { session: false }),
//   (req, res) => {
//     State.getStatebyName(req.body.state, (err, state) => {
//       if (err) {
//         res
//           .status(200)
//           .json({ success: false, msg: 'error occured', err: err });
//       } else {
//         if (state) {
//           filtercities = state.cities.filter(city => {
//             // console.log(city);
//             // console.log(req.body.city);
//             if (city !== req.body.city) {
//               return true;
//             }
//           });
//           state.cities = filtercities;
//           state.allotedBy = req.body.allotedBy;
//           State.addState(state, (err, state) => {
//             if (err) {
//               res
//                 .status(200)
//                 .json({ success: false, msg: 'error occured', err: err });
//             } else {
//               res.status(200).json({
//                 success: true,
//                 msg: 'city and states are assigned',
//                 state: state
//               });
//             }
//           });
//         } else {
//           res.status(200).json({ success: false, msg: 'state not found' });
//         }
//       }
//     });
//   }
// );
module.exports = router;
