const express = require('express');
const router = express.Router();
const passport = require('passport');
const OfficePackage = require('../models/office-package');
const OldOfficePackage = require('../models/oldOfficePackage');

router.post('/getPackageById', (req, res) => {
  const packageid = req.body.packageid;
  OfficePackage.getPackageById(packageid, (err, package) => {
    if(err){
      res.json({success: true, msg: JSON.stringify(err)});
    } else {
      if(package){
        res.json({success: true, msg: 'Package Fetched', package: package});
      } else {
        res.json({success: false, msg: 'Package not found'});
      }
    }
  });
});

// create office pacakge
router.post(
  '/create-office-package',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    OfficePackage.getPackageByName(req.body.packageName, (err, package) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'error occured',
          err: JSON.stringify(err)
        });
      } else {
        if (package) {
          res.status(200).json({
            success: true,
            msg: 'package name already exist',
            package: package
          });
        } else {
          let officePack = new OfficePackage({
            packageName: req.body.packageName,
            packageAmount: req.body.packageAmount,
            distributionCount: req.body.distributionCount,
            createdBy: req.body.createdBy
          });
          OfficePackage.addPackage(officePack, (err, pack) => {
            if (err) {
              res.status(200).json({
                success: false,
                msg: 'error occured',
                err: JSON.stringify(err)
              });
            } else {
              res.status(200).json({
                success: true,
                msg: 'package created successfully',
                package: pack
              });
            }
          });
        }
      }
    });
  }
);

// get all office-packages
router.get(
  '/get-office-packages',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    OfficePackage.getAllPackages((err, packs) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'error occured',
          err: JSON.stringify(err)
        });
      } else {
        res
          .status(200)
          .json({
            success: true,
            msg: 'office packages found',
            packages: packs
          });
      }
    });
  }
);

router.post('/edit-officePackage', passport.authenticate('jwt', { session: false }), (req, res) => {
  let package = req.body.package;
  OfficePackage.getPackageById(package._id, (err, pack) => {
    if (err) {
      res.json({ success: false, msg: 'Error in updating package', err: JSON.stringify(err) });
    } else {
      if (pack) {
        pack.packageAmount = package.packageAmount;
        pack.packageName = package.packageName;
        pack.distributionCount = package.distributionCount;
        OfficePackage.addPackage(pack, (err, updatedPack) => {
          if (err) {
            res.json({ success: false, msg: 'Error in updating package ', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'Office package updated successfully', package: updatedPack });
          }
        });
      } else {
        res.json({ success: false, msg: 'Package not found' });
      }
    }
  });
});

router.post('/delete-officePackage', passport.authenticate('jwt', { session: false }), (req, res) => {
  OfficePackage.getPackageByName(req.body.packageName, (err, package) => {
    if (err) {
      res.json({ success: false, msg: 'Error in deletion of package', err: JSON.stringify(err) });
    } else {
      if (package) {
        let newPack = new OldOfficePackage({
          packageName: package.packageName,
          packageAmount: package.packageAmount,
          distributionCount: package.distributionCount,
          createdBy: package.createdBy
        });

        OldOfficePackage.addPackage(newPack, (err, pack) => {
          if (err) {
            res.json({ success: false, msg: 'Error in deletion of package', err: JSON.stringify(err) });
          } else {
            OfficePackage.deletePackage(package._id, (err, deletedPack) => {
              if (err) {
                res.json({ success: false, msg: 'Error in deletion of package', err: JSON.stringify(err) });
              } else {
                res.json({ success: true, msg: 'package is deleted successfully', package: deletedPack });
              }
            });
          }
        });
      } else {
        res.json({ success: false, msg: 'Package not found' });
      }
    }
  });
});

module.exports = router;
