const express = require('express');
const router = express.Router();
const config = require('../config/database');
const OfficePack = require('../models/office-pack');
const passport = require('passport');

router.post(
  '/create-officePack',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let officePack = new OfficePack({
        name: req.body.name,
        amount: req.body.amount,
        remark: req.body.remark,
        distributionCount: req.body.distributionCount,
        topOfficePack: req.body.topOfficePack,
        commission: req.body.commission,
        createdBy: req.body.createdBy,
    });

    OfficePack.getOfficePackbyName(officePack.name, (err, foundOfficePack) => {
      if (err) {
        res.json({ success: false, msg: 'Error in fetching officePack', err: JSON.stringify(err) });
      } else {
        if (foundOfficePack) {
          res.json({ success: false, msg: 'OfficePack already exist with this OfficePack Name.' });
        } else {
            OfficePack.addOfficePack(officePack, (err, newOfficePack) => {
              if (err) {
                res.status(200).json({ success: false, msg: 'Internal Server Error', err: err });
              } else {
                res.status(200).json({
                  success: true,
                  msg: 'OfficePack added successfully',
                  officePack: newOfficePack
                });
              }
            });
        }
      }
    });
  }
);

router.get(
  '/getAllOfficePacks',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    OfficePack.getAllOfficePacks((err, officePacksList) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch officePacks list.',
          err: err
        });
      } else {
        res.status(200).json({
          success: true,
          msg: 'OfficePack list fetched',
          officePacksList: officePacksList
        });
      }
    });
  }
);

router.post(
  '/getOfficePackById',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    OfficePack.getOfficePackById(req.body.id, (err, officePack) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch officePack details',
          err: err
        });
      } else {
        if(officePack){
          res
          .status(200)
          .json({ success: true, msg: 'officePack fetched', officePack: officePack });
        } else {
          res.json({ success: false,  msg: 'OfficePack not found'});
        }
      }
    });
  }
);


router.post('/edit-officePack', passport.authenticate('jwt', { session: false }), (req, res) => {
  let officePack = req.body.officePack;
//   console.log(officePacks);
  OfficePack.getOfficePackById(officePack._id, (err, foundOfficePack) => {
    if (err) {
      res.json({ success: false, msg: 'Error in updating officePack', err: JSON.stringify(err) });
    } else {
      if (foundOfficePack) {
        foundOfficePack.name = officePack.name;
        foundOfficePack.remark = officePack.remark;
        foundOfficePack.topOfficePack = officePack.topOfficePack;
        foundOfficePack.permissions = officePack.permissions;
        foundOfficePack.commission = officePack.commission;
        foundOfficePack.updatedBy = officePack.updatedBy;
        foundOfficePack.updatedAt = Date.now();

        OfficePack.addOfficePack(foundOfficePack, (err, updatedOfficePack) => {
          if (err) {
            res.json({ success: false, msg: 'Error in editing officePack', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'OfficePack edited successfully', officePack: updatedOfficePack });
          }
        });
      } else {
        res.json({ success: false, msg: 'officePack not found' });
      }
    }
  });
});



router.post('/delete-officePack', passport.authenticate('jwt', { session: false }), (req, res) => {
  OfficePack.getOfficePackbyName(req.body.officePackName, (err, foundOfficePack) => {
    if (err) {
      res.json({ success: false, msg: 'Error in deletion of officePack', err: JSON.stringify(err) });
    } else {
      if (foundOfficePack) {
        OfficePack.deleteOfficePack(foundOfficePack._id, (err, deletedOfficePack) => {
          if (err) {
            res.json({ success: false, msg: 'Error in deletion of officePack', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'OfficePack deleted successfully ' });
          }
        });
      } else {
        res.json({ success: false, msg: 'OfficePack not found' });
      }
    }
  });
});

module.exports = router;
