const express = require('express');
const router = express.Router();
const passport = require('passport');
const Office = require('../models/office');
const config = require('../config/database');
const OldOffice = require('../models/oldOffice');

router.post(
  '/create-office',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let office = new Office({
      officeName: req.body.officeName,
      officeLocation: req.body.officeLocation,
      officeLocationType: req.body.officeLocationType,
      officeAddress: req.body.officeAddress,
      officeContact: req.body.officeContact,
      officeCity: req.body.officeCity,
      officeAdmin: req.body.officeAdmin,
      officeType: req.body.officeType,
      officePackage: req.body.officePackage,
      officePercentage: req.body.officePercentage,
      topOffice: req.body.topOffice,
      createdBy: req.body.createdBy
    });

    Office.addOffice(office, (err, newOffice) => {
      if (err) {
        res.status(200).json({ success: false, msg: '', err: err });
      } else {
        res.status(200).json({
          success: true,
          msg: 'Office added successfully',
          office: newOffice
        });
      }
    });
  }
);

router.get(
  '/getAllOffice',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Office.getAllOffice((err, officeList) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch office list.',
          err: err
        });
      } else {
        // console.log(officeList);
        res.status(200).json({
          success: true,
          msg: 'Office list fetched',
          officeList: officeList
        });
      }
    });
  }
);

router.post(
  '/getOfficeById',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.body);
    Office.getOfficeById(req.body.id, (err, office) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch office details',
          err: err
        });
      } else {
        if (!office) {
          res.json({ success: false, msg: "Office Not Found" });
        } else {
          res
            .status(200)
            .json({ success: true, msg: 'office fetched', office: office });
        }
      }
    });
  }
);

router.post(
  '/getOfficesByTopOffice',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.body);
    Office.getOfficesByTopOffice(req.body.topOffice, (err, offices) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch office details',
          err: err
        });
      } else {
        if (!offices) {
          res.json({ success: false, msg: "Offices Not Found" });
        } else {
          res
            .status(200)
            .json({ success: true, msg: 'offices fetched', offices: offices });
        }
      }
    });
  }
);

router.post(
  '/getOfficesByPackage',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.body);
    Office.getOfficesByPackage(req.body.officePackage, (err, offices) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch office details',
          err: err
        });
      } else {
        if (!offices) {
          res.json({ success: false, msg: "Offices Not Found" });
        } else {
          res
            .status(200)
            .json({ success: true, msg: 'offices fetched', offices: offices });
        }
      }
    });
  }
);

router.post(
  '/getOfficesByTopOfficeAndPackage',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.body);
    Office.getOfficesByTopOfficeAndPackage(req.body.officePackage, req.body.topPackage, (err, offices) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch office details',
          err: err
        });
      } else {
        if (!offices) {
          res.json({ success: false, msg: "Offices Not Found" });
        } else {
          res
            .status(200)
            .json({ success: true, msg: 'offices fetched', offices: offices });
        }
      }
    });
  }
);

router.post('/edit-office', passport.authenticate('jwt', { session: false }), (req, res) => {
  let office = req.body.office;
  Office.getOfficeById(office._id, (err, foundOffice) => {
    if (err) {
      res.json({ success: false, msg: 'Error in editing office', err: JSON.stringify(err) });
    } else {
      if (foundOffice) {
        foundOffice.officeName = office.officeName;
        foundOffice.officeLocation = office.officeLocation;
        foundOffice.officeLocationType = office.officeLocationType;
        foundOffice.officeAddress = office.officeAddress;
        foundOffice.officeContact = office.officeContact;
        foundOffice.officeCity = office.officeCity;
        foundOffice.topOffice = office.topOffice;
        foundOffice.officePackage = office.officePackage;
        foundOffice.officePercentage = office.officePercentage;
        Office.addOffice(foundOffice, (err, updatedOffice) => {
          if (err) {
            res.json({ success: false, msg: 'Error in updating office', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'Office Updated successfully', office: updatedOffice });
          }
        });
      } else {
        res.json({ success: false, msg: 'Office not found' });
      }
    }
  });
});

router.post('/deleteOffice', passport.authenticate('jwt', { session: false }), (req, res) => {
  // console.log(req.body.officeName);
  Office.getOfficeByName(req.body.officeName, (err, office) => {
    if (err) {
      // console.log(err);
      res.json({ success: false, msg: 'Error in deleting package', err: JSON.stringify(err) });
    } else {
      if (office) {
        if (office.officeType == 'main') {
          res.json({ success: false, msg: 'Main office cant be deleted' })
        } else {
          let oldOff = new OldOffice({
            officeName: office.officeName,
            officeAddress: office.officeAddress,
            officeAdmin: office.officeAdmin,
            officeCity: office.officeCity,
            officeContact: office.officeContact,
            officeWallet: office.officeWallet,
            officePackage: office.officePackage,
            officeType: office.officeType,
            officePercentage: office.officePercentage,
            createdBy: office.createdBy
          });
          OldOffice.addOffice(oldOff, (err, addedoffice) => {
            if (err) {
              // console.log(err);
              res.json({ success: false, msg: 'error in deleting office', err: JSON.stringify(err) });
            } else {
              Office.deleteOffice(req.body.officeName, (err, officeDeleted) => {
                if (err) {
                  // console.log(err);
                  res.json({ success: false, msg: 'Error in deleting office', err: JSON.stringify(err) });
                } else {
                  res.json({ success: true, msg: 'office deleted successfully..', office: officeDeleted });
                }
              });
            }
          });
        }
      } else {
        res.json({ success: false, msg: 'Office not found' })
      }
    }
  });
});

module.exports = router;
