const express = require('express');
const router = express.Router();
const Remainder = require('../models/remainder');
const passport = require('passport');

router.get(
  '/get-remainders/:username',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.params.username);
    const username = req.params.username;
    Remainder.getRemainderByUsername(username, (err, remainders) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Error occured',
          err: JSON.stringify(err)
        });
      } else {
        res.status(200).json({
          success: true,
          msg: 'remainders found',
          remainders: remainders
        });
      }
    });
  }
);

router.post('/get-remaindersByUsernames',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const username = req.body.usernames;
    Remainder.getRemaindersByUsernames(username, (err, remainders) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        res.json({ success: true, remainders: remainders });
      }
    })
  });

router.post('/get-remaindersByUsernamesLimited',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const username = req.body.usernames;
    const limit = req.body.limit;
    const skip = req.body.skip;
    Remainder.getRemaindersByUsernamesLimited(username, limit, skip, (err, remainders) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        Remainder.getRemaindersByUsernamesCount(username, (err, count) => {
          if (err) {
            res.json({ success: false, msg: 'Internal server error' });
          } else {
            res.json({ success: true, remainders: remainders, count: count });
          }
        });
      }
    });
  });

router.post('/get-remaindersByUsernameByDateUnread',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const username = req.body.username;
    const date = req.body.date;
    Remainder.getRemaindersByUsernameByDateUnread(username, date, (err, remainders) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        res.json({ success: true, remainders: remainders });
      }
    })
  });

router.post('/get-remaindersByUsernamesByDateUnread',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const username = req.body.usernames;
    const date = req.body.date;
    Remainder.getRemaindersByUsernamesByDateUnread(username, date, (err, remainders) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        res.json({ success: true, remainders: remainders });
      }
    })
  });

router.post('/get-remaindersByUsernamesByDateRead',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const username = req.body.usernames;
    const date = req.body.date;
    Remainder.getRemaindersByUsernamesByDateRead(username, date, (err, remainders) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        res.json({ success: true, remainders: remainders });
      }
    })
  });

router.post('/get-remaindersByUsernameByDate',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const username = req.body.username;
    const date = req.body.date;
    Remainder.getRemaindersByUsernameByDateRead(username, date, (err, remainders) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        res.json({ success: true, remainders: remainders });
      }
    })
  });

router.post('/update-remainder',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    const id = req.body.id;
    Remainder.getRemainderById(id, (err, remainder) => {
      if (err) {
        res.json({ success: false, msg: 'Internal server error' });
      } else {
        if (!remainder) {
          res.json({ success: false, msg: 'Remainder not found' });
        } else {
          remainder.read = req.body.remainder.read;
          Remainder.addRemainder(remainder, (err, remainder) => {
            if (err) {
              res.json({ success: false, msg: 'Internal server error' });
            } else {
              res.json({ success: true, remainder: remainder });
            }
          })
        }
      }
    });
  });

module.exports = router;
