const express = require('express');
const router = express.Router();
const config = require('../config/database');
const siteInfo = require('../models/siteInfo');
const passport = require('passport');
const siteInfoController = require('./../controllers/siteInfoController');

router.post(
    '/create',
    passport.authenticate('jwt', { session: false }),
    siteInfoController.addSiteInfo
);

router.delete(
    '/remove',
    passport.authenticate('jwt', { session: false }),
    siteInfoController.delete
);

router.get(
    '/getData',
    passport.authenticate('jwt', { session: false }),
    siteInfoController.getDetails

);

router.put(
    '/updateData',
    passport.authenticate('jwt', { session: false }),
    siteInfoController.updateDetails

);

router.post(
    '/getPurposeById',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        Purpose.getPurposeById(req.body.purposeId, (err, purpose) => {
            if (err) {
                res.status(200).json({
                    success: false,
                    msg: 'Failed to fetch purpose details',
                    err: err
                });
            } else {
                if (purpose) {
                    res
                        .status(200)
                        .json({ success: true, msg: 'purpose fetched', purpose: purpose });
                } else {
                    res.json({ success: false, msg: 'Purpose Not Found' });
                }
            }
        });
    }
);


router.post('/edit-purpose', passport.authenticate('jwt', { session: false }), (req, res) => {
    let purpose = req.body.purpose;
    console.log(purpose);
    Purpose.getPurposeById(purpose._id, (err, foundPurpose) => {
        if (err) {
            res.json({ success: false, msg: 'Error in updating purpose', err: JSON.stringify(err) });
        } else {
            // console.log(req.body)
            Purpose.getPurposebyShortName(req.body.shortName, (err, ffoundSource) => {
                if (err) {
                    res.json({ success: false, msg: 'Error in updating purpose', err: JSON.stringify(err) });
                } else {
                    if (ffoundSource) {
                        res.json({ success: false, msg: 'Purpose is already exist with this shortName', err: JSON.stringify(err) });
                    } else {
                        if (foundPurpose) {
                            foundPurpose.purposeName = purpose.purposeName;
                            foundPurpose.purposeCode = purpose.purposeCode;
                            foundPurpose.purposeId = purpose.purposeId;
                            foundPurpose.shortName = purpose.shortName;
                            foundPurpose.show = purpose.show;

                            Purpose.addPurpose(foundPurpose, (err, updatedPurpose) => {
                                if (err) {
                                    res.json({ success: false, msg: 'Error in editing purpose', err: JSON.stringify(err) });
                                } else {
                                    res.json({ success: true, msg: 'Purpose edited successfully', purpose: updatedPurpose });
                                }
                            });
                        } else {
                            res.json({ success: false, msg: 'purpose not found' });
                        }
                    }
                }
            });
        }
    });
});

router.post('/delete-purpose', passport.authenticate('jwt', { session: false }), (req, res) => {
    Purpose.getPurposebyName(req.body.purposeName, (err, foundPurpose) => {
        if (err) {
            res.json({ success: false, msg: 'Error in deletion of purpose', err: JSON.stringify(err) });
        } else {
            if (foundPurpose) {
                let newP = new OldPurpose({
                    purposeName: foundPurpose.purposeName,
                    shortName: foundPurpose.shortName,
                    show: foundPurpose.show,
                    purposeCode: foundPurpose.purposeCode,
                    purposeId: foundPurpose.purposeId,
                    allotedBy: foundPurpose.allotedBy
                });
                OldPurpose.addPurpose(newP, (err, addedP) => {
                    if (err) {
                        res.json({ success: false, msg: 'Error in deletion purpose' });
                    } else {
                        Purpose.deletePurpose(foundPurpose.purposeName, (err, deletedPurpose) => {
                            if (err) {
                                res.json({ success: false, msg: 'Error in deletion of purpose', err: JSON.stringify(err) });
                            } else {
                                res.json({ success: true, msg: 'Purpose deleted successfully ' });
                            }
                        });
                    }
                });
            } else {
                res.json({ success: false, msg: 'Purpose not found' });
            }
        }
    });
});

module.exports = router;
