const _ = require('lodash');
const express = require('express');
const router = express.Router();
const passport = require('passport');
const User = require('../models/user');
const State = require('../models/state');
const Helpers = require('../Helpers/Helpers');
const AllStates = require('../models/state.js')
router.get(
  '/get-states',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    State.getAllStates((err, states) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'Something went wrong', err: err });
      } else {
        res
          .status(200)
          .json({ success: true, msg: 'states are found', states: states });
      }
    });
  }
);

router.post(
  '/get-states',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let country = req.body.country;
    // console.log(country);
    State.getStatebyCountry(country, (err, states) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'Something went wrong', err: err });
      } else {
        res
          .status(200)
          .json({ success: true, msg: 'states are found', states: states });
      }
    });
  }
);

router.post('/create-state', passport.authenticate('jwt', { session: false }), (req, res) => {
  State.getStatebyName(Helpers.firstUpper(req.body.stateName), (err, state) => {
    if (err) {
      res.json({ success: false, msg: 'Error in creating State', err: JSON.stringify(err) });
    } else {
      if (state) {
        res.json({ success: false, msg: 'State already exist' });
      } else {
        let newState = new State({
          stateName: Helpers.firstUpper(req.body.stateName),
          countryName: Helpers.firstUpper(req.body.countryName)
        });
        State.addState(newState, (err, createdState) => {
          if (err) {
            res.json({ success: false, msg: 'Error in creating State', err: JSON.stringify(err) });
          } else {
            if (createdState) {
              res.json({ success: true, msg: 'State Created Successfully ' });
            } else {
              res.json({ success: false, msg: 'Error in creating state ' });
            }
          }
        });
      }
    }
  });
});

router.get(
  '/get-citiesof/:state',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    State.getStatebyName(req.params.state, (err, state) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Error in fetching state cities',
          err: err
        });
      } else {
        res.status(200).json({
          success: true,
          msg: 'state cities are found',
          cities: state.cities
        });
      }
    });
  }
);

module.exports = router;
