const express = require('express');
const router = express.Router();
const passport = require("passport");
const Transaction = require('../models/transaction');
const NewMember = require('../models/new_member');
const rc = require('./../controllers/responseController');

router.post('/add',
  passport.authenticate("jwt", { session: false }),
  (req, res) => {
    transactionToAdd = req.body.transactionToAdd;
    const newTransaction = new Transaction(req.body.transactionToAdd);
    if (!newTransaction) {
      return rc.setResponse(res, {
        msg: 'No transaction to perform'
      });
    }
    Transaction.addTransaction(newTransaction, (err, doc) => {
      if (err) {
        return rc.setResponse(res, {
          msg: err.message
        })
      } else {
        let id;
        let transAmount;
        if (transactionToAdd.action === 'debit') {
          id = (transactionToAdd.payee).split('@')[1];
          transAmount = 0 - Math.abs(parseInt(transactionToAdd.amount));
        } else {
          id = (transactionToAdd.receiver).split('@')[1];
          transAmount = parseInt(transactionToAdd.amount);
        }
        console.log(transAmount);
        NewMember.updateWallet(id, transAmount, doc._id, (m_err, m_doc) => {
          if (m_err) {
            Transaction.deleteTransactionById(doc._id, (t_err, t_doc) => {
              if (t_err) {
                rc.setResponse(res, { msg: t_err.message });
              } else {
                rc.setResponse(res, { msg: m_err });
              }
            });
          } else {
            return rc.setResponse(res, {
              success: true,
              msg: 'Transaction Successfull',
              data: doc
            });
          }
        });
      }
    })
  });

router.get('/',
  passport.authenticate("jwt", { session: false }),
  (req, res) => {
    Transaction.getTransactions((err, docs) => {
      if (err) {
        return rc.setResponse(res, {
          msg: err.message
        })
      } else {
        return rc.setResponse(res, {
          success: true,
          msg: 'All Transactions Fetched',
          data: docs
        });
      }
    })
  });

router.get('/byId/:id',
  passport.authenticate("jwt", { session: false }),
  (req, res) => {
    const id = req.params.id;
    Transaction.getTransaction(id, (err, doc) => {
      if (err) {
        return rc.setResponse(res, {
          msg: err.message
        })
      } else {
        return rc.setResponse(res, {
          success: true,
          msg: 'Transaction Fetched',
          data: doc
        });
      }
    })
  });

router.post('/byfield',
  // passport.authenticate("jwt", { session: false }),
  (req, res) => {
    const fieldName = req.body.fieldName;
    const fieldValue = req.body.fieldValue;
    console.log(fieldName);
    console.log(fieldValue);

    Transaction.getTransactionByFieldName(fieldName, fieldValue, (err, docs) => {
      if (err) {
        return rc.setResponse(res, {
          msg: err.message
        })
      } else {
        return rc.setResponse(res, {
          success: true,
          msg: 'Transaction Fetched',
          data: docs
        });
      }
    })
  });

router.put('/update/:id',
  passport.authenticate("jwt", { session: false }),
  (req, res) => {
    Transaction.updateTransaction(req.params.id, req.body, (err, docs) => {
      if (err) {
        return rc.setResponse(res, {
          msg: err.message
        })
      } else {
        return rc.setResponse(res, {
          success: true,
          msg: 'Transaction Updated',
          data: docs
        });
      }
    })
  });

router.delete('/byid/:id',
  passport.authenticate("jwt", { session: false }),
  (req, res) => {
    Transaction.deleteTransactionById(req.params.id, (err, docs) => {
      if (err) {
        return rc.setResponse(res, {
          msg: err.message
        })
      } else {
        return rc.setResponse(res, {
          success: true,
          msg: 'Transaction Deleted',
          data: docs
        });
      }
    })
  });

module.exports = router;