import cv2
import os
import signal

count = 1
datasets = 'faces'
face_classifier = cv2.CascadeClassifier('haarcascade_frontalface_default.xml')

def face_extractor(img):
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    faces = face_classifier.detectMultiScale(gray, 1.3, 5)
    if len(faces) != 0:
        # Crop all faces found
        for (x, y, w, h) in faces:
            cropped_face = img[y:y+h, x:x+w]
            return cropped_face
    return None

def signal_handler(signal, frame):
    print('Keyboard interrupt detected. Exiting gracefully...')
    cv2.destroyAllWindows()
    os._exit(0)

signal.signal(signal.SIGINT, signal_handler)

try:
    for (subdirs, dirs, files) in os.walk(datasets):
        for subdir in dirs:
            subjectpath = os.path.join(datasets, subdir)
            for filename in os.listdir(subjectpath):
                path = os.path.join(subjectpath, filename)
                image = cv2.imread(path)
                cropped_face = face_extractor(image)
                if cropped_face is not None:
                    face = cv2.resize(cropped_face, (200, 200))
                    gray = cv2.cvtColor(face, cv2.COLOR_BGR2GRAY)
                    file_name_path = path  # Save processed image with the same filename
                    cv2.imwrite(file_name_path, gray)
                    count += 1
                else:
                    os.remove(path)

except SystemExit:
    pass
